@@ -32,6 +32,7 @@
 #include <X11/Xatom.h>
 
 #include "bg.h"
+#include "panel.h"
 
 //#define DEBUG
 #include "dbg.h"
@@ -61,9 +62,9 @@ struct _FbBg {
 static void fb_bg_class_init (FbBgClass *klass);
 static void fb_bg_init (FbBg *monitor);
 static void fb_bg_finalize (GObject *object);
-static Pixmap fb_bg_get_xrootpmap(FbBg *monitor);
 static void fb_bg_changed(FbBg *monitor);
-
+static Pixmap fb_bg_get_xrootpmap_real(FbBg *bg);
+static GdkFilterReturn fb_bg_event_filter(GdkXEvent *xevent, GdkEvent *event, FbBg *bg);
 
 static guint signals [LAST_SIGNAL] = { 0 };
 
@@ -124,7 +125,7 @@ fb_bg_init (FbBg *bg)
     bg->dpy = GDK_DISPLAY();
     bg->xroot = DefaultRootWindow(bg->dpy);
     bg->id = XInternAtom(bg->dpy, "_XROOTPMAP_ID", False);
-    bg->pixmap = fb_bg_get_xrootpmap(bg);
+    bg->pixmap = fb_bg_get_xrootpmap_real(bg);
     gcv.ts_x_origin = 0;
     gcv.ts_y_origin = 0;
     gcv.fill_style = FillTiled;
@@ -134,6 +135,8 @@ fb_bg_init (FbBg *bg)
         mask |= GCTile ;
     }
     bg->gc = XCreateGC (bg->dpy, bg->xroot, mask, &gcv) ;
+    gdk_window_add_filter(gdk_get_default_root_window (),
+          (GdkFilterFunc)fb_bg_event_filter, bg);
     RET();
 }
 
@@ -153,42 +156,71 @@ fb_bg_finalize (GObject *object)
     ENTER;
     bg = FB_BG (object);
     XFreeGC(bg->dpy, bg->gc);
+    gdk_window_remove_filter(gdk_get_default_root_window (),
+          (GdkFilterFunc)fb_bg_event_filter, bg);
+    default_bg = NULL;
+    
     RET();
 }
 
-
-static Pixmap
+Pixmap
 fb_bg_get_xrootpmap(FbBg *bg)
 {
-    Pixmap ret;
+    ENTER;
+    RET(bg->pixmap);
+}
+
+static Pixmap
+fb_bg_get_xrootpmap_real(FbBg *bg)
+{
+    Pixmap ret = None;
 
     ENTER;
-    ret = None;    
     if (bg->id) {
         int  act_format, c = 2 ;
         u_long  nitems ;
         u_long  bytes_after ;
-        u_char *prop ;
+        u_char *prop = NULL;
         Atom ret_type;
 
         do {
             if (XGetWindowProperty(bg->dpy, bg->xroot, bg->id, 0, 1,
                       False, XA_PIXMAP, &ret_type, &act_format,
                       &nitems, &bytes_after, &prop) == Success) {
-                if (prop && ret_type == XA_PIXMAP) {
+                if (ret_type == XA_PIXMAP) {
                     ret = *((Pixmap *)prop);
-                    XFree(prop);
-                    break;
+                    c = -c ; //to quit loop
                 }
+                XFree(prop);           
             }
         } while (--c > 0);
     }
-    //fprintf(stderr, "_XROOTPMAP_ID = 0x%x\n", ret);
     RET(ret);
 
 }
 
 
+
+GdkPixmap *
+fb_bg_get_xroot_pix_for_area(FbBg *bg, gint x, gint y, gint width, gint height, gint depth)
+{
+    GdkPixmap *gbgpix;
+    Pixmap bgpix;
+    
+    ENTER;
+    if (bg->pixmap == None)
+        RET(NULL);
+    gbgpix = gdk_pixmap_new(NULL, width, height, depth);
+    if (!gbgpix) {
+        ERR("gdk_pixmap_new failed\n");
+        RET(NULL);
+    }
+    bgpix =  gdk_x11_drawable_get_xid(gbgpix);
+    XSetTSOrigin(bg->dpy, bg->gc, -x, -y) ;
+    XFillRectangle(bg->dpy, bgpix, bg->gc, 0, 0, width, height);
+    RET(gbgpix);
+}
+
 GdkPixmap *
 fb_bg_get_xroot_pix_for_win(FbBg *bg, GtkWidget *widget)
 {
@@ -196,17 +228,20 @@ fb_bg_get_xroot_pix_for_win(FbBg *bg, GtkWidget *widget)
     Window dummy;
     Pixmap bgpix;
     GdkPixmap *gbgpix;
-    int  width, height ;
-    int  border, depth, x, y ;
+    guint  width, height, border, depth;
+    int  x, y;
 
     ENTER;
+    if (bg->pixmap == None)
+        RET(NULL);
     win =  GDK_WINDOW_XWINDOW(widget->window); 
-    if (!XGetGeometry(bg->dpy, win, &dummy, &x, &y, &width, &height, &border, &depth)) {
+    if (!XGetGeometry(bg->dpy, win, &dummy, &x, &y, &width, &height, &border,
+              &depth)) {
         DBG2("XGetGeometry failed\n");
         RET(NULL);
     }
     XTranslateCoordinates(bg->dpy, win, bg->xroot, 0, 0, &x, &y, &dummy);
-    DBG("win=%x %dx%d%+d%+d\n", win, width, height, x, y);
+    DBG("win=%lxx %dx%d%+d%+d\n", win, width, height, x, y);
     gbgpix = gdk_pixmap_new(NULL, width, height, depth);
     if (!gbgpix) {
         ERR("gdk_pixmap_new failed\n");
@@ -230,11 +265,14 @@ fb_bg_composite(GdkDrawable *base, GdkGC *gc, guint32 tintcolor, gint alpha)
     if (!cmap) {
         cmap = gdk_colormap_get_system ();
     }
+    DBG("here\n");
     ret = gdk_pixbuf_get_from_drawable (NULL, base, cmap, 0, 0, 0, 0, w, h);
     if (!ret)
         RET();
+    DBG("here w=%d h=%d\n", w, h);
     ret2 = gdk_pixbuf_composite_color_simple(ret, w, h, 
           GDK_INTERP_HYPER, 255-alpha, MIN(w, h), tintcolor, tintcolor);
+    DBG("here\n");
     if (!ret2) {
         g_object_unref(ret);
         RET();
@@ -251,7 +289,7 @@ static void
 fb_bg_changed(FbBg *bg)
 {
     ENTER;
-    bg->pixmap = fb_bg_get_xrootpmap(bg);
+    bg->pixmap = fb_bg_get_xrootpmap_real(bg);
     if (bg->pixmap != None) {
         XGCValues  gcv;
         
@@ -279,3 +317,18 @@ FbBg *fb_bg_get_for_display(void)
         g_object_ref(default_bg);
     RET(default_bg);
 }
+
+static GdkFilterReturn
+fb_bg_event_filter(GdkXEvent *xevent, GdkEvent *event, FbBg *bg)
+{
+    XEvent *ev = (XEvent *) xevent;
+
+    ENTER;
+    DBG("win = 0x%lx\n", ev->xproperty.window);
+    if (ev->type != PropertyNotify || ev->xproperty.window != GDK_ROOT_WINDOW()
+          || ev->xproperty.atom != a_XROOTPMAP_ID) 
+        RET(GDK_FILTER_CONTINUE);
+    
+    fb_bg_notify_changed_bg(bg);
+    RET(GDK_FILTER_REMOVE);
+}
