@@ -258,7 +258,7 @@ gtk_bgbox_size_allocate (GtkWidget *widget, GtkAllocation *wa)
     GtkBin *bin;
     GtkAllocation ca;
     GtkBgboxPrivate *priv;
-    int same_alloc;
+    int same_alloc, border;
   
     ENTER;
     same_alloc = !memcmp(&widget->allocation, wa, sizeof(*wa));
@@ -268,16 +268,11 @@ gtk_bgbox_size_allocate (GtkWidget *widget, GtkAllocation *wa)
           widget->allocation.width, widget->allocation.height);
     widget->allocation = *wa;
     bin = GTK_BIN (widget);
-
-    if (GTK_WIDGET_NO_WINDOW (widget)) {
-        ca.x = wa->x + GTK_CONTAINER (widget)->border_width;
-        ca.y = wa->y + GTK_CONTAINER (widget)->border_width;
-    } else {
-        ca.x = 0;
-        ca.y = 0;
-    }
-    ca.width  = MAX (wa->width  - GTK_CONTAINER (widget)->border_width * 2, 0);
-    ca.height = MAX (wa->height - GTK_CONTAINER (widget)->border_width * 2, 0);
+    border = GTK_CONTAINER (widget)->border_width;
+    ca.x = border;
+    ca.y = border;
+    ca.width  = MAX (wa->width  - border * 2, 0);
+    ca.height = MAX (wa->height - border * 2, 0);
  
     if (GTK_WIDGET_REALIZED (widget) && !GTK_WIDGET_NO_WINDOW (widget)
           && !same_alloc) {
@@ -361,11 +356,14 @@ gtk_bgbox_set_bg_root(GtkWidget *widget, GtkBgboxPrivate *priv)
     
     ENTER;
     priv->pixmap = fb_bg_get_xroot_pix_for_win(priv->bg, widget);
+    DBG("here\n");
     if (!priv->pixmap || priv->pixmap ==  GDK_NO_BG) {
         priv->bg_type = BG_NONE;
         RET();
     }
+    DBG("here\n");
     fb_bg_composite(priv->pixmap, widget->style->black_gc, priv->tintcolor, priv->alpha);
+    DBG("here\n");
     gdk_window_set_back_pixmap(widget->window, priv->pixmap, FALSE);
     //gdk_window_clear(widget->window);
     rect.x = widget->allocation.x;
