@@ -18,6 +18,7 @@
 //#define DEBUG
 #include "dbg.h"
 
+extern panel *the_panel;
 
 /* X11 data types */
 Atom a_UTF8_STRING;
@@ -38,6 +39,7 @@ Atom a_NET_CURRENT_DESKTOP;
 Atom a_NET_DESKTOP_NAMES;
 Atom a_NET_ACTIVE_WINDOW;
 Atom a_NET_CLOSE_WINDOW;
+Atom a_NET_SUPPORTED;
 Atom a_NET_WM_DESKTOP;
 Atom a_NET_WM_STATE;
 Atom a_NET_WM_STATE_SKIP_TASKBAR;
@@ -56,6 +58,7 @@ Atom a_NET_WM_WINDOW_TYPE_DIALOG;
 Atom a_NET_WM_WINDOW_TYPE_NORMAL;
 Atom a_NET_WM_DESKTOP;
 Atom a_NET_WM_NAME;
+Atom a_NET_WM_VISIBLE_NAME;
 Atom a_NET_WM_STRUT;
 Atom a_NET_WM_STRUT_PARTIAL;
 Atom a_NET_WM_ICON;
@@ -127,7 +130,7 @@ num2str(pair *p, int num, gchar *defval)
     RET(defval);
 }
 
-int
+extern  int
 get_line(FILE *fp, line *s)
 {
     gchar *tmp, *tmp2;
@@ -221,6 +224,7 @@ void resolve_atoms()
     a_NET_CURRENT_DESKTOP        = XInternAtom(GDK_DISPLAY(), "_NET_CURRENT_DESKTOP", False);
     a_NET_DESKTOP_NAMES          = XInternAtom(GDK_DISPLAY(), "_NET_DESKTOP_NAMES", False);
     a_NET_ACTIVE_WINDOW          = XInternAtom(GDK_DISPLAY(), "_NET_ACTIVE_WINDOW", False);
+    a_NET_SUPPORTED              = XInternAtom(GDK_DISPLAY(), "_NET_SUPPORTED", False);
     a_NET_WM_DESKTOP             = XInternAtom(GDK_DISPLAY(), "_NET_WM_DESKTOP", False);
     a_NET_WM_STATE               = XInternAtom(GDK_DISPLAY(), "_NET_WM_STATE", False);
     a_NET_WM_STATE_SKIP_TASKBAR  = XInternAtom(GDK_DISPLAY(), "_NET_WM_STATE_SKIP_TASKBAR", False);
@@ -240,6 +244,7 @@ void resolve_atoms()
     a_NET_WM_WINDOW_TYPE_NORMAL  = XInternAtom(GDK_DISPLAY(), "_NET_WM_WINDOW_TYPE_NORMAL", False);
     a_NET_WM_DESKTOP             = XInternAtom(GDK_DISPLAY(), "_NET_WM_DESKTOP", False);
     a_NET_WM_NAME                = XInternAtom(GDK_DISPLAY(), "_NET_WM_NAME", False);
+    a_NET_WM_VISIBLE_NAME        = XInternAtom(GDK_DISPLAY(), "_NET_WM_VISIBLE_NAME", False);
     a_NET_WM_STRUT               = XInternAtom(GDK_DISPLAY(), "_NET_WM_STRUT", False);
     a_NET_WM_STRUT_PARTIAL       = XInternAtom(GDK_DISPLAY(), "_NET_WM_STRUT_PARTIAL", False);
     a_NET_WM_ICON                = XInternAtom(GDK_DISPLAY(), "_NET_WM_ICON", False);
@@ -283,6 +288,81 @@ Xclimsgwm(Window win, Atom type, Atom arg)
     XSendEvent(GDK_DISPLAY(), win, False, 0L, (XEvent *) &xev);
 }
 
+
+void *
+get_utf8_property(Window win, Atom atom)
+{
+
+    Atom type;
+    int format;
+    gulong nitems;
+    gulong bytes_after;
+    gchar *val, *retval;
+    int result;
+    guchar *tmp = NULL;
+    
+    type = None;
+    retval = NULL;
+    result = XGetWindowProperty (GDK_DISPLAY(), win, atom, 0, G_MAXLONG, False,
+          a_UTF8_STRING, &type, &format, &nitems, 
+          &bytes_after, &tmp);
+    if (result != Success || type == None)
+        return NULL;
+    val = (gchar *) tmp;
+    if (val) {
+        if (type == a_UTF8_STRING && format == 8 && nitems != 0) 
+            retval = g_strndup (val, nitems);
+        XFree (val);
+    }
+    return retval;
+
+}
+
+char **
+get_utf8_property_list(Window win, Atom atom, int *count)
+{
+    Atom type;
+    int format, i;
+    gulong nitems;
+    gulong bytes_after;
+    gchar *s, **retval = NULL;
+    int result;
+    guchar *tmp = NULL;
+
+    *count = 0;
+    result = XGetWindowProperty(GDK_DISPLAY(), win, atom, 0, G_MAXLONG, False, 
+          a_UTF8_STRING, &type, &format, &nitems, 
+          &bytes_after, &tmp);  
+    if (result != Success || type != a_UTF8_STRING || tmp == NULL)
+        return NULL;
+
+    if (nitems) {
+        gchar *val = (gchar *) tmp;
+        DBG("res=%d(%d) nitems=%d val=%s\n", result, Success, nitems, val);
+        for (i = 0; i < nitems; i++) {
+            if (!val[i])
+                (*count)++;
+        }
+        retval = g_new0 (char*, *count + 2);
+        for (i = 0, s = val; i < *count; i++, s = s +  strlen (s) + 1) {
+            retval[i] = g_strdup(s);
+        }
+        if (val[nitems-1]) {
+            result = nitems - (s - val);
+            DBG("val does not ends by 0, moving last %d bytes\n", result);
+            g_memmove(s - 1, s, result);
+            val[nitems-1] = 0;
+            DBG("s=%s\n", s -1);
+            retval[i] = g_strdup(s - 1);
+            (*count)++;
+        }
+    }
+    XFree (tmp);
+  
+    return retval;
+
+}
+
 void *
 get_xaproperty (Window win, Atom prop, Atom type, int *nitems)
 {
@@ -357,7 +437,7 @@ int
 get_net_number_of_desktops()
 {
     int desknum;
-    unsigned long *data;
+    guint32 *data;
 
     ENTER;
     data = get_xaproperty (GDK_ROOT_WINDOW(), a_NET_NUMBER_OF_DESKTOPS,
@@ -375,7 +455,7 @@ int
 get_net_current_desktop ()
 {
     int desk;
-    unsigned long *data;
+    guint32 *data;
 
     ENTER;
     data = get_xaproperty (GDK_ROOT_WINDOW(), a_NET_CURRENT_DESKTOP, XA_CARDINAL, 0);
@@ -391,7 +471,7 @@ int
 get_net_wm_desktop(Window win)
 {
     int desk = 0;
-    unsigned long *data;
+    guint32 *data;
 
     ENTER;
     data = get_xaproperty (win, a_NET_WM_DESKTOP, XA_CARDINAL, 0);
@@ -417,26 +497,26 @@ get_net_wm_state(Window win, net_wm_state *nws)
     DBG( "%x: netwm state = { ", (unsigned int)win);
     while (--num3 >= 0) {
         if (state[num3] == a_NET_WM_STATE_SKIP_PAGER) {
-            DBG("NET_WM_STATE_SKIP_PAGER ");
+            DBGE("NET_WM_STATE_SKIP_PAGER ");
             nws->skip_pager = 1;
         } else if (state[num3] == a_NET_WM_STATE_SKIP_TASKBAR) {
-            DBG( "NET_WM_STATE_SKIP_TASKBAR ");	    
+            DBGE("NET_WM_STATE_SKIP_TASKBAR ");	    
 	    nws->skip_taskbar = 1;
 	} else if (state[num3] == a_NET_WM_STATE_STICKY) {
-            DBG( "NET_WM_STATE_STICKY ");
+            DBGE("NET_WM_STATE_STICKY ");
 	    nws->sticky = 1;
         } else if (state[num3] == a_NET_WM_STATE_HIDDEN) {
-            DBG( "NET_WM_STATE_HIDDEN ");
+            DBGE("NET_WM_STATE_HIDDEN ");
             nws->hidden = 1;
 	} else if (state[num3] == a_NET_WM_STATE_SHADED) {
-            DBG( "NET_WM_STATE_SHADED ");
+            DBGE("NET_WM_STATE_SHADED ");
             nws->shaded = 1;
 	} else {
-	    DBG( "... ");
+	    DBGE("... ");
 	}
     }
     XFree(state);
-    DBG( "}\n");
+    DBGE( "}\n");
     RET();
 }
 
@@ -693,36 +773,15 @@ gtk_image_new_from_file_scaled(const gchar *file, gint width,
       gint height, gboolean keep_ratio)
 {
     GtkWidget *img;
-    GdkPixbuf *pb, *pb_scaled;
-    gfloat w, h, rw, rh;
+    GdkPixbuf *pb;
 
     ENTER;
-    if (!g_file_test(file, G_FILE_TEST_EXISTS)) 
-        goto err;
-
-    if (!(pb = gdk_pixbuf_new_from_file(file, NULL)))
-        goto err;
-    
-    if (keep_ratio) {
-        w = gdk_pixbuf_get_width(pb);
-        h = gdk_pixbuf_get_height(pb);
-        rw = w / width;
-        rh = h / height;
-        if (rw > rh)
-            height = h / rw;
-        else
-            width =  w / rh;
-    }
-    pb_scaled = gdk_pixbuf_scale_simple(pb, width, height,
-                                        GDK_INTERP_BILINEAR);
-    img = gtk_image_new_from_pixbuf(pb_scaled);			
-    g_object_unref(pb);
-    g_object_unref(pb_scaled);
-    RET(img);
-
- err:
-    img = gtk_image_new_from_stock(GTK_STOCK_MISSING_IMAGE,
-                                   GTK_ICON_SIZE_BUTTON);
+    if ((pb = gdk_pixbuf_new_from_file_at_size(file, width, height, NULL))) {
+        img = gtk_image_new_from_pixbuf(pb);			
+        g_object_unref(pb);
+    } else 
+        img = gtk_image_new_from_stock(GTK_STOCK_MISSING_IMAGE,
+              GTK_ICON_SIZE_BUTTON);
     RET(img);
 }
 
@@ -779,7 +838,7 @@ fb_button_enter (GtkImage *widget, GdkEventCrossing *event)
 {
     GdkPixbuf *dark, *light;
     int i;
-    guint hicolor;
+    gulong hicolor;
     guchar *src, *up, extra[3];
  
     ENTER;
@@ -788,8 +847,10 @@ fb_button_enter (GtkImage *widget, GdkEventCrossing *event)
     light = g_object_get_data(G_OBJECT(widget), "light");
     dark = gtk_image_get_pixbuf(widget);
     if (!light) {
-        hicolor = (gint) g_object_get_data(G_OBJECT(widget), "hicolor");
-        light = gdk_pixbuf_add_alpha(dark, FALSE, 0, 0, 0);      
+        hicolor = (gulong) g_object_get_data(G_OBJECT(widget), "hicolor");
+        light = gdk_pixbuf_add_alpha(dark, FALSE, 0, 0, 0);
+        if (!light)
+            RET(TRUE);
         src = gdk_pixbuf_get_pixels (light);
         for (i = 2; i >= 0; i--, hicolor >>= 8)
             extra[i] = hicolor & 0xFF;
@@ -804,8 +865,6 @@ fb_button_enter (GtkImage *widget, GdkEventCrossing *event)
                     src[i] += extra[i];
             }
         }
-        if (!light)
-            RET(TRUE);
         g_object_set_data_full (G_OBJECT(widget), "light", light, g_object_unref);
     }
     g_object_ref(dark);
@@ -829,9 +888,43 @@ fb_button_leave (GtkImage *widget, GdkEventCrossing *event, gpointer user_data)
     RET(TRUE);    
 }
 
-
 GtkWidget *
-fb_button_new_from_file(gchar *fname, int width, int height, guint hicolor, gboolean keep_ratio)
+fb_button_new_from_icon_file(gchar *iname, gchar *fname, int width, int height,
+      gulong hicolor, gboolean keep_ratio)
+{
+    GtkWidget *b, *image;
+
+    ENTER;
+    DBG("iname = %s\n", iname);
+    DBG("fname = %s\n", fname);
+    //make background window
+    b = gtk_bgbox_new();
+    gtk_container_set_border_width(GTK_CONTAINER(b), 0);
+    GTK_WIDGET_UNSET_FLAGS (b, GTK_CAN_FOCUS);
+    //make image
+    image = fb_image_new_from_icon_file(iname, fname, width, height, keep_ratio);
+    gtk_container_add(GTK_CONTAINER(b), image);    
+    //set calbacks
+    gtk_misc_set_alignment(GTK_MISC(image), 0, 1);
+    g_object_set_data(G_OBJECT(image), "hicolor", (gpointer)hicolor);
+    gtk_misc_set_padding (GTK_MISC(image), 0, 0);
+    if (hicolor > 0) {
+        gtk_widget_add_events(b, GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK);
+        g_signal_connect_swapped (G_OBJECT (b), "enter-notify-event",
+              G_CALLBACK (fb_button_enter), image);
+        g_signal_connect_swapped (G_OBJECT (b), "leave-notify-event",
+              G_CALLBACK (fb_button_leave), image);
+    }
+
+    gtk_widget_show(image);
+    gtk_widget_show(b);
+    RET(b);
+}
+
+#if 0
+GtkWidget *
+fb_button_new_from_file(gchar *fname, int width, int height, gulong hicolor,
+      gboolean keep_ratio)
 {
     GtkWidget *b, *image;
     
@@ -840,6 +933,7 @@ fb_button_new_from_file(gchar *fname, int width, int height, guint hicolor, gboo
     gtk_container_set_border_width(GTK_CONTAINER(b), 0);
     GTK_WIDGET_UNSET_FLAGS (b, GTK_CAN_FOCUS);
     image = gtk_image_new_from_file_scaled(fname, width, height, keep_ratio);
+    gtk_misc_set_alignment(GTK_MISC(image), 0, 0);
     g_object_set_data(G_OBJECT(image), "hicolor", (gpointer)hicolor);
     gtk_misc_set_padding (GTK_MISC(image), 0, 0);
     if (hicolor > 0) {
@@ -854,24 +948,71 @@ fb_button_new_from_file(gchar *fname, int width, int height, guint hicolor, gboo
     gtk_widget_show(b);
     RET(b);
 }
+#endif
+GtkWidget *
+fb_image_new_from_icon_file(gchar *iname, gchar *fname, int width, int height,
+      gboolean keep_ratio)
+{
+    GtkWidget *image;
+    GdkPixbuf *pb = NULL;
+    
+    pb = fb_pixbuf_new_from_icon_file(iname, fname, width, height);
+    if (pb) {
+        image = gtk_image_new_from_pixbuf(pb);
+        DBG("px: w=%d h=%d\n", gdk_pixbuf_get_width(pb), gdk_pixbuf_get_height(pb));
+        g_object_unref(pb);
+    } else 
+        image = gtk_image_new_from_stock(GTK_STOCK_MISSING_IMAGE, GTK_ICON_SIZE_BUTTON);
+    DBG("image = %p\n", image);
+    RET(image);
+}
+
+GdkPixbuf *
+fb_pixbuf_new_from_icon_file(gchar *iname, gchar *fname, int width, int height)
+{
+    GdkPixbuf *pb = NULL;
+	
+    ENTER;
+    if (iname) {
+        GtkIconInfo *icon_info;
+        icon_info = gtk_icon_theme_lookup_icon(gtk_icon_theme_get_default(),
+              iname, MAX(width, height), 0);
+        if (icon_info) {
+            
+            DBG("iname = %s file = %s size = %d\n", iname,
+                  gtk_icon_info_get_filename(icon_info),
+                  MAX(width, height));
+            pb = gdk_pixbuf_new_from_file_at_size(gtk_icon_info_get_filename(icon_info),
+                  width, height, NULL);
+            
+            gtk_icon_info_free(icon_info);
+        } else
+            DBG("icon info failed\n");
+    }
+
+    if (fname && !pb) {
+        pb = gdk_pixbuf_new_from_file_at_size(fname, width, height, NULL);
+        DBG("%s\n", pb ? "from file" : "from stock");
+    }
+    RET(pb);
+}
 
 GtkWidget *
-fb_button_new_from_file_with_label(gchar *fname, int width, int height,
-      guint hicolor, gboolean keep_ratio, gchar *name)
+fb_button_new_from_icon_file_with_label(gchar *iname, gchar *fname, int width, int height,
+      gulong hicolor, gboolean keep_ratio, gchar *name)
 {
     GtkWidget *b, *image, *box, *label;
     
     ENTER;
     b = gtk_bgbox_new();
-    gtk_container_set_border_width(GTK_CONTAINER(b), 0);
-    GTK_WIDGET_UNSET_FLAGS (b, GTK_CAN_FOCUS);
 
     box = gtk_hbox_new(FALSE, 0);
     gtk_container_set_border_width(GTK_CONTAINER(box), 0);
     GTK_WIDGET_UNSET_FLAGS (box, GTK_CAN_FOCUS);
     gtk_container_add(GTK_CONTAINER(b), box);
-    
-    image = gtk_image_new_from_file_scaled(fname, width, height, keep_ratio);
+
+    DBG("here\n");
+    image = fb_image_new_from_icon_file(iname, fname, width, height, keep_ratio);
     g_object_set_data(G_OBJECT(image), "hicolor", (gpointer)hicolor);
     gtk_misc_set_padding (GTK_MISC(image), 0, 0);
     if (hicolor > 0) {
@@ -881,7 +1022,9 @@ fb_button_new_from_file_with_label(gchar *fname, int width, int height,
         g_signal_connect_swapped (G_OBJECT (b), "leave-notify-event",
               G_CALLBACK (fb_button_leave), image);
     }
+    DBG("here\n");
     gtk_box_pack_start(GTK_BOX(box), image, FALSE, FALSE, 0);
+    DBG("here\n");
     if (name) {
         label =  gtk_label_new(name);
         gtk_misc_set_padding(GTK_MISC(label), 2, 0);
@@ -890,3 +1033,52 @@ fb_button_new_from_file_with_label(gchar *fname, int width, int height,
     gtk_widget_show_all(b);
     RET(b);
 }
+
+
+
+void
+menu_pos(GtkMenu *menu, gint *x, gint *y, gboolean *push_in, GtkWidget *widget)
+{
+    int ox, oy, w, h;
+    
+    ENTER;
+    if (widget) {
+        gdk_window_get_origin(widget->window, &ox, &oy);
+        ox += widget->allocation.x;
+        oy += widget->allocation.y;
+    } else {
+        gdk_display_get_pointer(gdk_display_get_default(), NULL, &ox, &oy, NULL);
+        ox -= 20;
+        if (ox < 0)
+            ox = 0;
+        oy -= 10;
+        if (oy < 0)
+            oy = 0;
+    }
+    w = GTK_WIDGET(menu)->requisition.width;
+    h = GTK_WIDGET(menu)->requisition.height;
+    if (the_panel->orientation == ORIENT_HORIZ) {
+        // x
+        *x = ox;
+        if (*x + w > gdk_screen_width())
+            *x = gdk_screen_width() - w;
+        // y
+        if (the_panel->edge == EDGE_TOP)
+            *y = the_panel->ah;
+        else
+            *y = gdk_screen_height() - the_panel->ah - h;
+    } else {
+        // x
+        if (the_panel->edge == EDGE_LEFT)
+            *x = the_panel->aw;
+        else
+            *x = gdk_screen_width() - the_panel->aw - w;
+        // y
+        *y = oy;
+        if (*y + h > gdk_screen_height())
+            *y = gdk_screen_height() - h;
+    }
+    DBG("w-h %d %d\n", w, h);
+    *push_in = TRUE;
+    RET();
+}
