@@ -21,18 +21,18 @@ static gchar *cfgfile = NULL;
 static gchar version[] = VERSION;
 gchar *cprofile = "default";
 
-int verbose;
 int config = 0;
 FbEv *fbev;
-
+gint force_quit = 0;
 //#define DEBUG
 #include "dbg.h"
 
+int log_level;
 
 FILE *pconf; // plugin part of profile file
 
 panel *p;
-
+panel *the_panel;
 
 /****************************************************
  *         panel's handlers for WM events           *
@@ -50,7 +50,7 @@ panel_del_wm_strut(panel *p)
 static void
 panel_set_wm_strut(panel *p)
 {
-    unsigned long data[12] = { 0 };
+    gulong data[12] = { 0 };
     int i = 4;
 
     ENTER;
@@ -143,17 +143,15 @@ panel_event_filter(GdkXEvent *xevent, GdkEvent *event, panel *p)
             DBG("A_NET_NUMBER_OF_DESKTOPS\n");
             p->desknum = get_net_number_of_desktops();
             fb_ev_trigger(fbev, EV_NUMBER_OF_DESKTOPS);
-	} else if (at == a_NET_ACTIVE_WINDOW) {
+	} else if (at == a_NET_DESKTOP_NAMES) {
+            DBG("A_NET_DESKTOP_NAMES\n");
+            fb_ev_trigger(fbev, EV_DESKTOP_NAMES);
+        } else if (at == a_NET_ACTIVE_WINDOW) {
             DBG("A_NET_ACTIVE_WINDOW\n");
             fb_ev_trigger(fbev, EV_ACTIVE_WINDOW);
-        } else if (at == a_NET_CLIENT_LIST_STACKING) {
+        }else if (at == a_NET_CLIENT_LIST_STACKING) {
             DBG("A_NET_CLIENT_LIST_STACKING\n");
             fb_ev_trigger(fbev, EV_CLIENT_LIST_STACKING);
-        } else if (at == a_XROOTPMAP_ID) {
-            DBG("a_XROOTPMAP_ID\n");
-            if (p->transparent) {
-                fb_bg_notify_changed_bg(p->bg);
-            }
 	} else if (at == a_NET_WORKAREA) {
             DBG("A_NET_WORKAREA\n");
             p->workarea = get_xaproperty (GDK_ROOT_WINDOW(), a_NET_WORKAREA, XA_CARDINAL, &p->wa_len);
@@ -183,7 +181,7 @@ panel_destroy_event(GtkWidget * widget, GdkEvent * event, gpointer data)
 {
     //panel *p = (panel *) data;
 
-    ENTER2;  
+    ENTER;  
     //if (!p->self_destroy)
     gtk_main_quit();
     RET(FALSE);
@@ -216,42 +214,40 @@ panel_size_req(GtkWidget *widget, GtkRequisition *req, panel *p)
 static gint
 panel_size_alloc(GtkWidget *widget, GtkAllocation *a, panel *p)
 {
-    GtkAllocation *aa = &widget->allocation;
+    static gint x, y;
     
     ENTER;
-    DBG("installed alloc: size (%d, %d). pos (%d, %d)\n", aa->width, aa->height, aa->x, aa->y);
-    DBG("suggested alloc: size (%d, %d). pos (%d, %d)\n", a->width, a->height, a->x, a->y);
-    DBG("prev pref alloc: size (%d, %d). pos (%d, %d)\n", p->aw, p->ah, p->ax, p->ay);
+    DBG("gtk: size (%d, %d). pos (%d, %d)\n", a->width, a->height, a->x, a->y);
+    DBG("our: size (%d, %d). pos (%d, %d)\n", p->aw, p->ah, p->ax, p->ay);
     if (p->widthtype == WIDTH_REQUEST)
         p->width = (p->orientation == ORIENT_HORIZ) ? a->width : a->height;
     if (p->heighttype == HEIGHT_REQUEST)
         p->height = (p->orientation == ORIENT_HORIZ) ? a->height : a->width;
     calculate_position(p);
-    DBG("curr pref alloc: size (%d, %d). pos (%d, %d)\n", p->aw, p->ah, p->ax, p->ay);
-    if (a->width == p->aw && a->height == p->ah && a->x == p->ax && a->y == p ->ay) {
+    DBG("our: size (%d, %d). pos (%d, %d)\n", p->aw, p->ah, p->ax, p->ay);
+    if (a->width == p->aw && a->height == p->ah && x == p->ax && y == p ->ay) {
         DBG("actual coords eq to preffered. just returning\n");
         RET(TRUE);
     }
-
-    /*
-    if (p->setstrut) {
-        panel_del_wm_strut(p);
-        gtk_window_move(GTK_WINDOW(p->topgwin), p->ax, p->ay);
-        panel_set_wm_strut(p);
-    } else {
-        gtk_window_move(GTK_WINDOW(p->topgwin), p->ax, p->ay);
-        }
-    */
+    x = p->ax;
+    y = p->ay;
     gtk_window_move(GTK_WINDOW(p->topgwin), p->ax, p->ay);
-    //gdk_window_move_resize (p->topgwin->window, p->ax, p->ay, p->aw, p->ah);
     DBG("moving to %d %d\n", p->ax, p->ay);
-    //gdk_flush();
     if (p->setstrut)
         panel_set_wm_strut(p);
-    //gdk_window_clear(p->topgwin->window);
     RET(TRUE);
 }
 
+static void
+panel_style_set(GtkWidget *widget, GtkStyle  *previous_style, panel *p)
+{
+    ENTER;
+    if (GTK_WIDGET_REALIZED(widget)) {
+        DBG("realized\n");
+        gtk_main_quit();
+    }
+    RET();
+}
 
 static  gboolean
 panel_configure_event (GtkWidget *widget, GdkEventConfigure *e, panel *p)
@@ -326,7 +322,7 @@ panel_start_gui(panel *p)
 {
     Atom state[3];
     XWMHints wmhints;
-    unsigned int val;
+    guint32 val;
  
     
     ENTER;
@@ -352,7 +348,9 @@ panel_start_gui(panel *p)
           (GCallback) panel_configure_event, p);
     g_signal_connect (G_OBJECT (p->topgwin), "realize",
           (GCallback) panel_realize, p);
-     
+    g_signal_connect (G_OBJECT (p->topgwin), "style-set",
+          (GCallback) panel_style_set, p);
+    
     gtk_widget_realize(p->topgwin);
     //gdk_window_set_decorations(p->topgwin->window, 0);
     gtk_widget_set_app_paintable(p->topgwin, TRUE);
@@ -375,7 +373,7 @@ panel_start_gui(panel *p)
     if (p->round_corners)
         make_round_corners(p);
  
-    p->box = p->my_box_new(FALSE, 1); 
+    p->box = p->my_box_new(FALSE, p->spacing); 
     gtk_container_set_border_width(GTK_CONTAINER(p->box), 0);
     gtk_box_pack_start(GTK_BOX(p->lbox), p->box, TRUE, TRUE, 0);
     gtk_widget_show(p->box);
@@ -451,6 +449,8 @@ panel_parse_global(panel *p, FILE *fp)
                 p->heighttype = str2num(height_pair, s.t[1], HEIGHT_NONE);
             } else if (!g_ascii_strcasecmp(s.t[0], "height")) {
                 p->height = atoi(s.t[1]);
+            } else if (!g_ascii_strcasecmp(s.t[0], "spacing")) {
+                p->spacing = atoi(s.t[1]);
             } else if (!g_ascii_strcasecmp(s.t[0], "SetDockType")) {
                 p->setdocktype = str2num(bool_pair, s.t[1], 0);
             } else if (!g_ascii_strcasecmp(s.t[0], "SetPartialStrut")) {
@@ -514,7 +514,7 @@ panel_parse_plugin(panel *p, FILE *fp)
     plugin *plug = NULL;
     gchar *type = NULL;
     FILE *tmpfp;
-    int expand , padding;
+    int expand , padding, border;
     
     ENTER;
     s.len = 256;
@@ -522,7 +522,7 @@ panel_parse_plugin(panel *p, FILE *fp)
         ERR( "can't open temporary file with tmpfile()\n");
         RET(0);
     }
-    expand = padding = 0;
+    border = expand = padding = 0;
     while (get_line(fp, &s) != LINE_BLOCK_END) {
         if (s.type == LINE_NONE) {
             ERR( "fbpanel: bad line %s\n", s.str);
@@ -536,6 +536,8 @@ panel_parse_plugin(panel *p, FILE *fp)
                 expand = str2num(bool_pair,  s.t[1], 0);
             else if (!g_ascii_strcasecmp(s.t[0], "padding"))
                 padding = atoi(s.t[1]);
+            else if (!g_ascii_strcasecmp(s.t[0], "border"))
+                border = atoi(s.t[1]);
             else {
                 ERR( "fbpanel: unknown var %s\n", s.t[0]);
                 goto error;
@@ -573,6 +575,7 @@ panel_parse_plugin(panel *p, FILE *fp)
     plug->fp = tmpfp;
     plug->expand = expand;
     plug->padding = padding;
+    plug->border = border;
     fprintf(tmpfp, "}\n");
     fseek(tmpfp, 0, SEEK_SET);
     DBG("starting\n");
@@ -582,6 +585,7 @@ panel_parse_plugin(panel *p, FILE *fp)
     }
     DBG("plug %s\n", type);
     p->plugins = g_list_append(p->plugins, plug);
+    g_free(type);
     RET(1);
     
  error:
@@ -616,6 +620,7 @@ panel_start(panel *p, FILE *fp)
     p->transparent = 0;
     p->alpha = 127;
     p->tintcolor = 0xFFFFFFFF;
+    p->spacing = 0;
     fbev = fb_ev_new();
     if ((get_line(fp, &s) != LINE_BLOCK_START) || g_ascii_strcasecmp(s.t[0], "Global")) {
         ERR( "fbpanel: config file must start from Global section\n");
@@ -685,7 +690,7 @@ usage()
     printf("Command line options:\n");
     printf(" --help      -- print this help and exit\n");
     printf(" --version   -- print version and exit\n");
-    printf(" --verbose   -- print usefull messages at runtime\n");
+    printf(" --log <number> -- set log level 0-5. 0 - none 5 - chatty\n");
     printf(" --configure -- launch configuration utility\n");
     printf(" --profile name -- use specified profile\n");
     printf("\n");
@@ -703,11 +708,16 @@ open_profile(gchar *profile)
     FILE *fp;
 
     ENTER;
+    LOG(LOG_INFO, "loading %s profile\n", profile);
+    if (cfgfile) {
+        g_free(cfgfile);
+        cfgfile = NULL;
+    }
     fname = g_strdup_printf("%s/.fbpanel/%s", getenv("HOME"), profile);
-    if ((fp = fopen(fname, "r"))) {
+    fp = fopen(fname, "r");
+    LOG(LOG_INFO, "   %s %s\n", fname, fp ? "ok" : "no");
+    if (fp) {
         cfgfile = fname;
-        if (verbose)
-            ERR("Using %s\n", fname);
         RET(fp);
     }
     //ERR("Can't load %s\n", fname);
@@ -715,15 +725,15 @@ open_profile(gchar *profile)
     
     /* check private configuration directory */
     fname = g_strdup_printf("%s/share/fbpanel/%s", PREFIX, profile);
-    if ((fp = fopen(fname, "r"))) {
+    fp = fopen(fname, "r");
+    LOG(LOG_INFO, "   %s %s\n", fname, fp ? "ok" : "no");
+    if (fp) {
         cfgfile = fname;
-        if (verbose)
-            ERR("Using %s\n", fname);
         RET(fp);
     }
     //ERR("Can't load %s\n", fname);
     g_free(fname);
-    ERR("Can't open '%s' profile\n", profile);
+    LOG(LOG_ERR, "Can't open '%s' profile\n", profile);
     RET(NULL);
 }
 
@@ -733,9 +743,9 @@ handle_error(Display * d, XErrorEvent * ev)
     char buf[256];
 
     ENTER;
-    if (verbose) {
+    if (log_level >= LOG_WARN) {
         XGetErrorText(GDK_DISPLAY(), ev->error_code, buf, 256);
-        ERR( "fbpanel : X error: %s\n", buf);
+        LOG(LOG_WARN, "fbpanel : X error: %s\n", buf);
     }
     RET();
 }
@@ -752,16 +762,21 @@ int
 main(int argc, char *argv[], char *env[])
 {
     int i;
-    int quit = 0;
     void configure();
     FILE *pfp; /* current profile FP */
-    
+    GdkPixbuf* icon;
     ENTER;
+    //printf("sizeof(gulong)=%d\n", sizeof(gulong));
     setlocale(LC_CTYPE, "");
     gtk_set_locale();
     gtk_init(&argc, &argv);
     XSetLocaleModifiers("");
     XSetErrorHandler((XErrorHandler) handle_error);
+    icon = gdk_pixbuf_new_from_file(IMGPREFIX "star.png", NULL);
+    if (icon) {
+        gtk_window_set_default_icon(icon);
+        g_object_unref(icon);
+    }
     resolve_atoms();
     for (i = 1; i < argc; i++) {
         if (!strcmp(argv[i], "-h") || !strcmp(argv[i], "--help")) {
@@ -770,8 +785,15 @@ main(int argc, char *argv[], char *env[])
         } else if (!strcmp(argv[i], "-v") || !strcmp(argv[i], "--version")) {
             printf("fbpanel %s\n", version);
             exit(0);
-        } else if (!strcmp(argv[i], "--verbose")) {
-            verbose = 1;
+        } else if (!strcmp(argv[i], "--log")) {
+            i++;
+            if (i == argc) {
+                ERR( "fbpanel: missing log level\n");
+                usage();
+                exit(1);
+            } else {
+                log_level = atoi(argv[i]);
+            }
         } else if (!strcmp(argv[i], "--configure") || !strcmp(argv[i], "-C")) {
             config = 1;
         } else if (!strcmp(argv[i], "--profile") || !strcmp(argv[i], "-p")) {
@@ -789,13 +811,14 @@ main(int argc, char *argv[], char *env[])
             exit(1);
         }
     }
+
+    gtk_icon_theme_append_search_path(gtk_icon_theme_get_default(),
+	IMGPREFIX);       
     signal(SIGUSR1, sig_usr);
     do {
-        if (verbose)
-            ERR("starting panel\n");
         if (!(pfp = open_profile(cprofile)))
             exit(1);
-        p = g_new0(panel, 1);
+        the_panel = p = g_new0(panel, 1);
         g_return_val_if_fail (p != NULL, 1);
         if (!panel_start(p, pfp)) {
             ERR( "fbpanel: can't start panel\n");
@@ -807,7 +830,7 @@ main(int argc, char *argv[], char *env[])
         gtk_main();
         panel_stop(p);
         g_free(p);
-    } while (quit == 0);
+    } while (force_quit == 0);
     
     exit(0);
 }
