@@ -60,13 +60,24 @@ typedef struct launchbar {
 static gboolean
 my_button_pressed(GtkWidget *widget, GdkEventButton *event, btn *b )
 {
+    GtkWidget *image;
+
     ENTER;
+    image = gtk_bin_get_child(GTK_BIN(widget));
     g_assert(b != NULL);
-    if ((event->type == GDK_BUTTON_RELEASE)
-          && (event->x >=0 && event->x < widget->allocation.width)
-          && (event->y >=0 && event->y < widget->allocation.height)) {
-        g_spawn_command_line_async(b->action, NULL);
+    if (event->type == GDK_BUTTON_RELEASE) {
+        if ((event->x >=0 && event->x < widget->allocation.width)
+              && (event->y >=0 && event->y < widget->allocation.height)) {
+            
+            g_spawn_command_line_async(b->action, NULL);
+        }
+        gtk_misc_set_padding (GTK_MISC(image), 0, 0);
+        
         //system(b->action);
+    } else if (event->type == GDK_BUTTON_PRESS) {
+      
+        gtk_misc_set_padding (GTK_MISC(image), 0, 3);
+        //ERR("here\n");
     }
     RET(TRUE);
 }
@@ -104,8 +115,8 @@ drag_data_received_cb (GtkWidget        *widget,
         RET();
     if (info == TARGET_URILIST) {
         DBG("uri drag received: info=%d len=%d data=%s\n", info, sd->length, sd->data);
-        s = sd->data;
-        end = sd->data + sd->length;
+        s = (gchar *)sd->data;
+        end = s + sd->length;
         str = g_strdup(b->action);
         while (s < end) {
             while (s < end && g_ascii_isspace(*s))
@@ -140,7 +151,7 @@ static int
 read_button(plugin *p)
 {
     launchbar *lb = (launchbar *)p->priv;
-    gchar *fname, *tooltip, *action;
+    gchar *iname, *fname, *tooltip, *action;
     //GdkPixbuf *gp, *gps;
     GtkWidget *button;
     line s;
@@ -155,7 +166,7 @@ read_button(plugin *p)
         RET(0);
     }
 
-    tooltip = fname = action = 0;
+    iname = tooltip = fname = action = 0;
     while (get_line(p->fp, &s) != LINE_BLOCK_END) {
         if (s.type == LINE_NONE) {
             ERR( "launchbar: illegal token %s\n", s.str);
@@ -166,8 +177,10 @@ read_button(plugin *p)
                 fname = expand_tilda(s.t[1]);
             else if (!g_ascii_strcasecmp(s.t[0], "tooltip"))
                 tooltip = g_strdup(s.t[1]);
+            else if (!g_ascii_strcasecmp(s.t[0], "icon"))
+                iname = g_strdup(s.t[1]);
             else if (!g_ascii_strcasecmp(s.t[0], "action"))
-                action = g_strdup(s.t[1]);
+                action = expand_tilda(s.t[1]);
             else {
                 ERR( "launchbar: unknown var %s\n", s.t[0]);
                 goto error;
@@ -178,21 +191,28 @@ read_button(plugin *p)
         }
     }
     DBG("action=%s\n", action);
-
+    //gtk_icon_theme_lookup_icon(lb->it, in->data, 40, GTK_ICON_LOOKUP_FORCE_SVG)));
     // button
     if (p->panel->orientation == ORIENT_HORIZ) {
-        w = 10000;
+        w = -1;
         //h = GTK_WIDGET(p->panel->box)->allocation.height;
         h = p->panel->ah;
     } else {
         //w = GTK_WIDGET(p->panel->box)->allocation.width;
         w = p->panel->aw;
-        h = 10000;
+        h = -1;
     }
-    button = fb_button_new_from_file(fname, w, h, 0x202020, TRUE);
-    gtk_container_set_border_width(GTK_CONTAINER(button), 0);
+    w = h = lb->iconsize;
+    //button = fb_button_new_from_file(iname, fname, w, h, 0x202020, TRUE);
+    button = fb_button_new_from_icon_file(iname, fname, w, h, 0x202020, TRUE);
+    //gtk_container_set_border_width(GTK_CONTAINER(button), 0);
     g_signal_connect (G_OBJECT (button), "button-release-event",
           G_CALLBACK (my_button_pressed), (gpointer) &lb->btns[lb->btn_num]);
+    g_signal_connect (G_OBJECT (button), "button-press-event",
+          G_CALLBACK (my_button_pressed), (gpointer) &lb->btns[lb->btn_num]);
+
+    DBG("here\n");
+    
     GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
     // DnD support
     gtk_drag_dest_set (GTK_WIDGET(button),
@@ -202,7 +222,7 @@ read_button(plugin *p)
     g_signal_connect (G_OBJECT(button), "drag_data_received",
           G_CALLBACK (drag_data_received_cb),  (gpointer) &lb->btns[lb->btn_num]);
 
- 
+    DBG("here\n");
 
     gtk_box_pack_start(GTK_BOX(lb->box), button, FALSE, FALSE, 0);
     gtk_widget_show(button);
@@ -210,8 +230,11 @@ read_button(plugin *p)
 
     if (p->panel->transparent) 
         gtk_bgbox_set_background(button, BG_ROOT, p->panel->tintcolor, p->panel->alpha);
-    
+
+    DBG("here\n");
     g_free(fname);
+    g_free(iname);
+    DBG("here\n");
     // tooltip
     if (tooltip) {
         gtk_tooltips_set_tip(GTK_TOOLTIPS (lb->tips), button, tooltip, NULL);
@@ -236,7 +259,7 @@ launchbar_constructor(plugin *p)
 {
     launchbar *lb; 
     line s;
-    GtkRequisition req;
+    //GtkRequisition req;
     static gchar *launchbar_rc = "style 'launchbar-style'\n"
         "{\n"
         "GtkWidget::focus-line-width = 0\n"
@@ -249,12 +272,12 @@ launchbar_constructor(plugin *p)
     ENTER;
     gtk_widget_set_name(p->pwid, "launchbar");
     gtk_rc_parse_string(launchbar_rc);
-    get_button_spacing(&req, GTK_CONTAINER(p->pwid), "");
+    //get_button_spacing(&req, GTK_CONTAINER(p->pwid), "");
     
     lb = g_new0(launchbar, 1);
     g_return_val_if_fail(lb != NULL, 0);
     p->priv = lb;
-    lb->box = p->panel->my_box_new(FALSE, 0);
+    lb->box = p->panel->my_box_new(FALSE, 1);
     gtk_container_add(GTK_CONTAINER(p->pwid), lb->box);
     gtk_container_set_border_width (GTK_CONTAINER (lb->box), 0);
     gtk_widget_show(lb->box);
@@ -264,9 +287,9 @@ launchbar_constructor(plugin *p)
         lb->iconsize = GTK_WIDGET(p->panel->box)->allocation.height;
     else
         lb->iconsize = GTK_WIDGET(p->panel->box)->allocation.width;
-    DBG("button: req width=%d height=%d\n", req.width, req.height);            
+    //DBG("button: req width=%d height=%d\n", req.width, req.height);            
     DBG("iconsize=%d\n", lb->iconsize);
-    
+
     s.len = 256;
     while (get_line(p->fp, &s) != LINE_BLOCK_END) {
         if (s.type == LINE_NONE) {
@@ -288,8 +311,25 @@ launchbar_constructor(plugin *p)
             goto error;
         }
     }
- 
 
+    if (0) {
+        GtkIconTheme* it = gtk_icon_theme_get_default();
+        GList* in = gtk_icon_theme_list_icons(it, NULL);
+        GList *tmp;
+        while ((tmp = in)) {
+            printf("%s %s\n", (char *)in->data, gtk_icon_info_get_filename(
+                       gtk_icon_theme_lookup_icon(it, in->data, 40, GTK_ICON_LOOKUP_FORCE_SVG)));
+         
+            in = g_list_next(in);
+            //g_free(tmp->data);
+            //g_list_free(tmp);
+        }
+        fflush(stdout);
+    }
+
+
+    
+    
     RET(1);
 
  error:
