@@ -44,7 +44,8 @@ extern panel *p;
 /* managed window: all related info that wm holds about its managed windows */
 typedef struct task {
     Window win;
-    int x, y, w, h;
+    int x, y;
+    guint w, h;
     gint refcount;
     guint stacking;
     guint desktop;
@@ -62,23 +63,28 @@ typedef struct _pager  pager;
 /* map of a desktop */
 struct _desk {
     GtkWidget *da;
+    Pixmap xpix;
+    GdkPixmap *gpix;
     GdkPixmap *pix;
     int no, dirty, first;
     gfloat scalew, scaleh;
-    pager *pg;
+    pager *pg; 
 };
 
 struct _pager {
-    GtkWidget *box, *eb;
+    GtkWidget *box;
     desk *desks[MAX_DESK_NUM];
     guint desknum;
     guint curdesk;
-    int dw, dh;
-    gfloat scalex, scaley, ratio;
+    gint wallpaper;
+    //int dw, dh;
+    gfloat /*scalex, scaley, */ratio;
     Window *wins;
     int winnum, dirty;
     GHashTable* htable;
     task *focusedtask;
+    FbBg *fbbg;
+    gint dah, daw;
 };
 
 
@@ -89,10 +95,15 @@ struct _pager {
 //if (t->nws.skip_pager || t->nwwt.desktop /*|| t->nwwt.dock || t->nwwt.splash*/ )
    
 static void pager_rebuild_all(FbEv *ev, pager *pg);
+static void desk_draw_bg(pager *pg, desk *d1);
+//static void pager_paint_frame(pager *pg, gint no, GtkStateType state);
+
+static void pager_destructor(plugin *p);
 
 static inline void desk_set_dirty_by_win(pager *p, task *t);
 static inline void desk_set_dirty(desk *d);
 static inline void desk_set_dirty_all(pager *pg);
+
 /*
 static void desk_clear_pixmap(desk *d);
 static gboolean task_remove_stale(Window *win, task *t, pager *p);
@@ -115,7 +126,7 @@ task_remove_stale(Window *win, task *t, pager *p)
         desk_set_dirty_by_win(p, t);
         if (p->focusedtask == t)
             p->focusedtask = NULL;
-        DBG("del %x\n", t->win);
+        DBG("del %lx\n", t->win);
         g_free(t);
         return TRUE;
     }
@@ -135,7 +146,8 @@ static void
 task_get_sizepos(task *t)
 {
     Window root, junkwin;
-    int dummy, rx, ry;
+    int rx, ry;
+    guint dummy;
     XWindowAttributes win_attributes;
     
     ENTER;
@@ -144,7 +156,7 @@ task_get_sizepos(task *t)
                   &dummy, &dummy)) {
             t->x = t->y = t->w = t->h = 2;
         }
-            
+        
     } else {
         XTranslateCoordinates (GDK_DISPLAY(), t->win, win_attributes.root, 
               -win_attributes.border_width,
@@ -154,7 +166,7 @@ task_get_sizepos(task *t)
         t->y = ry;
         t->w = win_attributes.width;
         t->h = win_attributes.height;
-        DBG("win=0x%x WxH=%dx%d\n", t->win,t->w, t->h);
+        DBG("win=0x%lx WxH=%dx%d\n", t->win,t->w, t->h);
     }
     RET();
 }
@@ -178,8 +190,9 @@ task_update_pix(task *t, desk *d)
     x = (gfloat)t->x * d->scalew;
     y = (gfloat)t->y * d->scaleh;
     w = (gfloat)t->w * d->scalew;
-    h = (t->nws.shaded) ? 2 : (gfloat)t->h * d->scaleh;
-    if (w < 2 || h < 2)
+    //h = (gfloat)t->h * d->scaleh;
+    h = (t->nws.shaded) ? 3 : (gfloat)t->h * d->scaleh;
+    if (w < 3 || h < 3)
         RET();
     widget = GTK_WIDGET(d->da);
     gdk_draw_rectangle (d->pix, 
@@ -211,15 +224,103 @@ desk_clear_pixmap(desk *d)
     if (!d->pix)
         RET();
     widget = GTK_WIDGET(d->da);
-    gdk_draw_rectangle (d->pix,
-          ((d->no == d->pg->curdesk) ? 
-                widget->style->dark_gc[GTK_STATE_SELECTED] :
-                widget->style->dark_gc[GTK_STATE_NORMAL]),
-          TRUE, 
-          0, 0,
-          widget->allocation.width,
-          widget->allocation.height);
+    if (d->pg->wallpaper && d->xpix != None) {
+    	gdk_draw_drawable (d->pix, 
+              widget->style->dark_gc[GTK_STATE_NORMAL], 
+              d->gpix, 
+              0, 0, 0, 0,
+              widget->allocation.width,
+              widget->allocation.height);      
+    } else {
+        gdk_draw_rectangle (d->pix,
+              ((d->no == d->pg->curdesk) ? 
+                    widget->style->dark_gc[GTK_STATE_SELECTED] :
+                    widget->style->dark_gc[GTK_STATE_NORMAL]),
+              TRUE, 
+              0, 0,
+              widget->allocation.width,
+              widget->allocation.height);
+    }
+    if (d->pg->wallpaper && d->no == d->pg->curdesk)
+        gdk_draw_rectangle (d->pix,
+              widget->style->light_gc[GTK_STATE_SELECTED], 
+              FALSE, 
+              0, 0,
+              widget->allocation.width -1,
+              widget->allocation.height -1);
+    RET();
+}
+
+
+static void
+desk_draw_bg(pager *pg, desk *d1)
+{
+    Pixmap xpix;
+    GdkPixmap *gpix; 
+    GdkPixbuf *p1, *p2;
+    gint width, height, depth;
+    FbBg *bg = pg->fbbg;
+    GtkWidget *widget = d1->da;
     
+    ENTER;
+    if (d1->no) {
+        desk *d0 = d1->pg->desks[0];
+        if (d0->gpix && d0->xpix != None
+              && d0->da->allocation.width == widget->allocation.width
+              && d0->da->allocation.height == widget->allocation.height) {
+            gdk_draw_drawable(d1->gpix,
+                  widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
+                  d0->gpix,0, 0, 0, 0,
+                  widget->allocation.width, 
+                  widget->allocation.height);
+            d1->xpix = d0->xpix;
+            DBG("copy gpix from d0 to d%d\n", d1->no);
+            RET();
+        }
+    }
+    xpix = fb_bg_get_xrootpmap(bg);
+    d1->xpix = None;
+    width = widget->allocation.width;
+    height = widget->allocation.height;
+    DBG("w %d h %d\n", width, height);
+    if (width < 3 || height < 3)
+        RET();
+    
+    // create new pix
+    xpix = fb_bg_get_xrootpmap(bg);
+    if (xpix == None)
+        RET();
+    depth = gdk_drawable_get_depth(widget->window);
+    gpix = fb_bg_get_xroot_pix_for_area(bg, 0, 0, gdk_screen_width(), gdk_screen_height(), depth);
+    if (!gpix) {
+        ERR("fb_bg_get_xroot_pix_for_area failed\n");
+        RET();
+    }
+    p1 = gdk_pixbuf_get_from_drawable(NULL, gpix, NULL, 0, 0, 0, 0,
+          gdk_screen_width(), gdk_screen_height());
+    if (!p1) {
+        ERR("gdk_pixbuf_get_from_drawable failed\n");
+        goto err_gpix;
+    }
+    p2 = gdk_pixbuf_scale_simple(p1, width, height,
+          //GDK_INTERP_NEAREST
+          //GDK_INTERP_TILES
+          //GDK_INTERP_BILINEAR
+          GDK_INTERP_HYPER
+        );
+    if (!p2) {
+        ERR("gdk_pixbuf_scale_simple failed\n");
+        goto err_p1;
+    }
+    gdk_draw_pixbuf(d1->gpix, widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
+          p2, 0, 0, 0, 0, width, height,  GDK_RGB_DITHER_NONE, 0, 0);
+
+    d1->xpix = xpix;
+    g_object_unref(p2);
+ err_p1:
+    g_object_unref(p1);
+ err_gpix:
+    g_object_unref(gpix);
     RET();
 }
 
@@ -286,36 +387,30 @@ desk_expose_event (GtkWidget *widget, GdkEventExpose *event, desk *d)
     RET(FALSE);
 }
 
+
 /* Upon realize and every resize creates a new backing pixmap of the appropriate size */
 static gint
 desk_configure_event (GtkWidget *widget, GdkEventConfigure *event, desk *d)
 {
     int w, h;
+    
     ENTER;
-    DBG("d->no=%d %dx%d\n", d->no, widget->allocation.width, widget->allocation.height);
+    w = widget->allocation.width;
+    h = widget->allocation.height;
+    
+    DBG("d->no=%d %dx%d %dx%d\n", d->no, w, h, d->pg->daw, d->pg->dah);
     if (d->pix)
         g_object_unref(d->pix);
-    
-    d->pix = gdk_pixmap_new(widget->window,
-          widget->allocation.width,
-          widget->allocation.height,
-          -1);    
-
-    d->scalew = (gfloat)widget->allocation.height / (gfloat)gdk_screen_height();
-    d->scaleh = (gfloat)widget->allocation.width  / (gfloat)gdk_screen_width();
-    desk_set_dirty(d);
-
-    //request best size
-    if (p->orientation != ORIENT_HORIZ) {
-        w = widget->allocation.width;
-        h = (gfloat) w / d->pg->ratio;
-    } else {
-        h = widget->allocation.height;
-        w = (gfloat) h * d->pg->ratio;
+    if (d->gpix)
+        g_object_unref(d->gpix);
+    d->pix = gdk_pixmap_new(widget->window, w, h, -1);
+    if (d->pg->wallpaper) {
+        d->gpix = gdk_pixmap_new(widget->window, w, h, -1);
+        desk_draw_bg(d->pg, d);
     }
-    DBG("requesting %dx%d\n", w, h);
-    gtk_widget_set_size_request(widget, w, h);
-    
+    d->scalew = (gfloat)h / (gfloat)gdk_screen_height();
+    d->scaleh = (gfloat)w / (gfloat)gdk_screen_width();
+    desk_set_dirty(d);
     RET(FALSE);
 }
 
@@ -375,7 +470,7 @@ desk_new(pager *pg, int i)
     d->no = i;
     
     d->da = gtk_drawing_area_new();
-    //gtk_widget_set_size_request(GTK_WIDGET(d->da), 10, 10);
+    gtk_widget_set_size_request(d->da, pg->daw, pg->dah);
     gtk_box_pack_start(GTK_BOX(pg->box), d->da, TRUE, TRUE, 0);
     gtk_widget_add_events (d->da, GDK_EXPOSURE_MASK
           | GDK_BUTTON_PRESS_MASK
@@ -390,10 +485,7 @@ desk_new(pager *pg, int i)
          (GCallback) desk_button_press_event, (gpointer)d);
     //g_signal_connect (G_OBJECT (d->da), "button_release_event",
     //     (GCallback) desk_button_release_event, (gpointer)d);
-    gtk_widget_show(d->da);
-    DBG("before pack\n");
-
-    DBG("after show\n");
+    gtk_widget_show_all(d->da);
     RET();
 }
 
@@ -408,6 +500,8 @@ desk_free(pager *pg, int i)
           i, d->no, d->da, d->pix);
     if (d->pix)
         g_object_unref(d->pix);
+    if (d->gpix)
+        g_object_unref(d->gpix);
     gtk_widget_destroy(d->da);
     g_free(d);
     RET();
@@ -426,7 +520,7 @@ do_net_active_window(FbEv *ev, pager *p)
 
     ENTER;
     fwin = get_xaproperty(GDK_ROOT_WINDOW(), a_NET_ACTIVE_WINDOW, XA_WINDOW, 0);
-    DBG("win=%x\n", fwin ? *fwin : 0);
+    DBG("win=%lx\n", fwin ? *fwin : 0);
     if (fwin) {
         t = g_hash_table_lookup(p->htable, fwin);
         if (t != p->focusedtask) {
@@ -447,14 +541,18 @@ do_net_active_window(FbEv *ev, pager *p)
 }
 
 static void
-do_net_current_desktop(FbEv *ev, pager *p)
+do_net_current_desktop(FbEv *ev, pager *pg)
 {
     ENTER;
-    desk_set_dirty(p->desks[p->curdesk]);
-    p->curdesk =  get_net_current_desktop ();
-    if (p->curdesk >= p->desknum)
-        p->curdesk = 0;
-    desk_set_dirty(p->desks[p->curdesk]);
+    desk_set_dirty(pg->desks[pg->curdesk]);
+    gtk_widget_set_state(pg->desks[pg->curdesk]->da, GTK_STATE_NORMAL);
+    //pager_paint_frame(pg, pg->curdesk, GTK_STATE_NORMAL);
+    pg->curdesk =  get_net_current_desktop ();
+    if (pg->curdesk >= pg->desknum)
+        pg->curdesk = 0;
+    desk_set_dirty(pg->desks[pg->curdesk]);
+    gtk_widget_set_state(pg->desks[pg->curdesk]->da, GTK_STATE_SELECTED);
+    //pager_paint_frame(pg, pg->curdesk, GTK_STATE_SELECTED);
     RET();
 }
 
@@ -493,7 +591,7 @@ do_net_client_list_stacking(FbEv *ev, pager *p)
             if (!FBPANEL_WIN(t->win))
                 XSelectInput (GDK_DISPLAY(), t->win, PropertyChangeMask | StructureNotifyMask);
             g_hash_table_insert(p->htable, &t->win, t);
-            DBG("add %x\n", t->win);
+            DBG("add %lx\n", t->win);
             desk_set_dirty_by_win(p, t);
         }
     }
@@ -531,7 +629,7 @@ pager_configurenotify(pager *p, XEvent *ev)
  
     if (!(t = g_hash_table_lookup(p->htable, &win))) 
         RET();
-    DBG("win=0x%x\n", win);
+    DBG("win=0x%lx\n", win);
     task_get_sizepos(t);
     desk_set_dirty_by_win(p, t);
     RET();
@@ -549,7 +647,7 @@ pager_propertynotify(pager *p, XEvent *ev)
     if ((win == GDK_ROOT_WINDOW()) || !(t = g_hash_table_lookup(p->htable, &win)))
         RET();
       
-    DBG("window=0x%x\n", t->win);
+    DBG("window=0x%lx\n", t->win);
     if (at == a_WM_STATE)    {
         DBG("event=WM_STATE\n");
         t->ws = get_wm_state (t->win);
@@ -577,9 +675,54 @@ pager_event_filter( XEvent *xev, GdkEvent *event, pager *pg)
         pager_configurenotify(pg, xev);
     RET(GDK_FILTER_CONTINUE);
 }
+#if 0
+static void
+pager_paint_frame(pager *pg, gint no, GtkStateType state)
+{
+    gint x, y, w, h, border;
 
+    ENTER;
+    RET();
+    //desk_set_dirty(pg->desks[no]);
+    border = gtk_container_get_border_width(GTK_CONTAINER(pg->box));
+    w = pg->box->allocation.width;
+    h = pg->desks[0]->da->allocation.height + border;
+    x = 0;
+    y = h * no;
+    h += border;
+    DBG("%d: %d %d %d %d\n", no, x, y, w, h);
+    gtk_paint_flat_box(pg->box->style, pg->box->window,
+          state, 
+          GTK_SHADOW_NONE,
+          NULL, pg->box, "box",
+          x + 1, y + 1,w, h);
+    RET();
+ 
+}
 
+static gint
+pager_expose_event (GtkWidget *widget, GdkEventExpose *event, pager *pg)
+{
+    ENTER;
+    DBG("curdesk=%d\n",  pg->curdesk);
+    pager_paint_frame(pg, pg->curdesk, GTK_STATE_SELECTED);
+    RET(TRUE);
+}
+#endif
 
+static void
+pager_bg_changed(FbBg *bg, pager *pg)
+{
+    int i;
+    
+    ENTER;
+    for (i = 0; i < pg->desknum; i++) {
+        desk *d = pg->desks[i];
+        desk_draw_bg(pg, d);
+        desk_set_dirty(d);
+    }
+    RET();
+}
 
 
 static void
@@ -616,15 +759,18 @@ pager_rebuild_all(FbEv *ev, pager *pg)
         for (i = desknum; i < pg->desknum; i++) 
             desk_new(pg, i);
     }
+    do_net_current_desktop(NULL, pg);
     do_net_client_list_stacking(NULL, pg);
+ 
     RET();
 }
 
-
+#define BORDER 1
 static int
 pager_constructor(plugin *plug)
 {
     pager *pg;
+    line s;
     
     ENTER;
     pg = g_new0(pager, 1);
@@ -632,24 +778,45 @@ pager_constructor(plugin *plug)
     plug->priv = pg;
 
     pg->htable = g_hash_table_new (g_int_hash, g_int_equal);
-
     pg->box = plug->panel->my_box_new(TRUE, 1);
     gtk_container_set_border_width (GTK_CONTAINER (pg->box), 0);
     gtk_widget_show(pg->box);
-  
+    /*
+    g_signal_connect (G_OBJECT (pg->box), "expose_event",
+       (GCallback) pager_expose_event, (gpointer)pg);
+    */
     gtk_bgbox_set_background(plug->pwid, BG_STYLE, 0, 0);
-    gtk_container_set_border_width (GTK_CONTAINER (plug->pwid), 1);
+    gtk_container_set_border_width (GTK_CONTAINER (plug->pwid), BORDER);
     gtk_container_add(GTK_CONTAINER(plug->pwid), pg->box);
-    pg->eb = pg->box;
 
     pg->ratio = (gfloat)gdk_screen_width() / (gfloat)gdk_screen_height();
-    pg->scaley = (gfloat)pg->dh / (gfloat)gdk_screen_height();
-    pg->scalex = (gfloat)pg->dw / (gfloat)gdk_screen_width();
-
-
+    if (plug->panel->orientation == ORIENT_HORIZ) {
+        pg->dah = plug->panel->ah - 2 * BORDER;
+        pg->daw = (gfloat) pg->dah * pg->ratio;
+    } else {
+        pg->daw = plug->panel->aw - 2 * BORDER;
+        pg->dah = (gfloat) pg->daw / pg->ratio;
+    }
+    pg->wallpaper = 1;
+    //pg->scaley = (gfloat)pg->dh / (gfloat)gdk_screen_height();
+    //pg->scalex = (gfloat)pg->dw / (gfloat)gdk_screen_width();
+    s.len = 256; 
+    while (get_line(plug->fp, &s) != LINE_BLOCK_END) {
+        if (s.type == LINE_VAR) {
+            if (!g_ascii_strcasecmp(s.t[0], "showwallpaper")) {
+                pg->wallpaper = str2num(bool_pair, s.t[1], 1);
+                break;
+            }
+        }
+        ERR( "pager: illegal in this context %s\n", s.str);
+        goto error;        
+    }
+    if (pg->wallpaper) {
+        pg->fbbg = fb_bg_get_for_display();
+        DBG("get fbbg %p\n", pg->fbbg);
+        g_signal_connect(G_OBJECT(pg->fbbg), "changed", G_CALLBACK(pager_bg_changed), pg);
+    }
     pager_rebuild_all(fbev, pg);
-    //do_net_current_desktop(fbev, pg);
-    //do_net_client_list_stacking(fbev, pg);
     
     gdk_window_add_filter(NULL, (GdkFilterFunc)pager_event_filter, pg );
   
@@ -662,6 +829,10 @@ pager_constructor(plugin *plug)
     g_signal_connect (G_OBJECT (fbev), "client_list_stacking",
           G_CALLBACK (do_net_client_list_stacking), (gpointer) pg);
     RET(1);
+
+ error:
+    pager_destructor(plug);
+    RET(0);
 }
 
 static void
@@ -680,7 +851,13 @@ pager_destructor(plugin *p)
     }
     g_hash_table_foreach_remove(pg->htable, (GHRFunc) task_remove_all, (gpointer)pg);
     g_hash_table_destroy(pg->htable);
-    gtk_widget_destroy(pg->eb);
+    gtk_widget_destroy(pg->box);
+    if (pg->wallpaper) {
+        g_signal_handlers_disconnect_by_func(G_OBJECT (pg->fbbg),
+              pager_bg_changed, pg);
+        DBG("put fbbg %p\n", pg->fbbg);
+        g_object_unref(pg->fbbg);
+    }
     g_free(pg);
     RET();
 }
