@@ -19,6 +19,11 @@
 #include "icon.xpm"
 #include "gtkbar.h"
 
+/*
+ * 2006.09.10 modified by Hong Jen Yee (PCMan) pcman.tw (AT) gmail.com
+ * Following features are added:
+ * 1. Add XUrgencyHint support. (Flashing task bar buttons, can be disabled)
+ */
 
 //#define DEBUG
 #include "dbg.h"
@@ -41,9 +46,13 @@ typedef struct _task{
     int desktop;
     net_wm_state nws;
     net_wm_window_type nwwt;
+    guint flash_timeout;
     unsigned int focused:1;
     unsigned int iconified:1;
+    unsigned int urgency:1;
     unsigned int using_netwm_icon:1;
+    unsigned int flash:1;
+    unsigned int flash_state:1;
 } task;
 
 
@@ -64,6 +73,7 @@ typedef struct _taskbar{
     int vis_task_num;
     int req_width;
     int hbox_width;
+    int spacing;
     int cur_desk;
     task *focused;
     task *ptk;
@@ -71,7 +81,8 @@ typedef struct _taskbar{
     char **desk_names;
     int desk_namesno;
     int desk_num;
-  
+    guint dnd_activate;
+    
     unsigned int iconsize;
     unsigned int task_width_max;
     unsigned int accept_skip_pager : 1;
@@ -80,6 +91,8 @@ typedef struct _taskbar{
     unsigned int show_all_desks : 1;
     unsigned int tooltips : 1;
     unsigned int icons_only : 1;
+    unsigned int use_mouse_wheel : 1;
+    unsigned int use_urgency_hint : 1;
 } taskbar;
 
 
@@ -94,6 +107,12 @@ static gchar *taskbar_rc = "style 'taskbar-style'\n"
 "}\n"
 "widget '*.taskbar.*' style 'taskbar-style'";
 
+static gboolean use_net_active=FALSE;
+
+#define DRAG_ACTIVE_DELAY	1000
+ 
+
+
 #define TASK_WIDTH_MAX   200
 #define TASK_PADDING     4
 static void tk_display(taskbar *tb, task *tk);
@@ -101,6 +120,12 @@ static void tb_propertynotify(taskbar *tb, XEvent *ev);
 static GdkFilterReturn tb_event_filter( XEvent *, GdkEvent *, taskbar *);
 static void taskbar_destructor(plugin *p);
 
+static gboolean tk_has_urgency( task* tk );
+
+static void tk_flash_window( task *tk );
+static void tk_unflash_window( task *tk );
+static void tk_raise_window( task *tk, guint32 time );
+extern panel *the_panel;
 
 #define TASK_VISIBLE(tb, tk) \
  ((tk)->desktop == (tb)->cur_desk || (tk)->desktop == -1 /* 0xFFFFFFFF */ )
@@ -109,22 +134,20 @@ static int
 task_visible(taskbar *tb, task *tk)
 {
     ENTER;
-    if (tk->desktop != -1 && !tb->show_all_desks && tk->desktop != tb->cur_desk)
-        RET(0);
-    if (tk->iconified) {
-        if (!tb->show_iconified)
-            RET(0);
-    } else {
-        if (!tb->show_mapped)
-            RET(0);
-    }
-    RET(1);
+    DBG("%lx: %d desktop=%d iconified=%d \n", tk->win, ret, tk->desktop, tk->iconified);
+    RET( (tb->show_all_desks || tk->desktop == -1 || (tk->desktop == tb->cur_desk))
+          && ((tk->iconified && tb->show_iconified) || (!tk->iconified && tb->show_mapped)) );
 }
 
 static int
 accept_net_wm_state(net_wm_state *nws, int accept_skip_pager)
 {
     ENTER;
+    DBG("accept_skip_pager=%d  skip_taskbar=%d skip_pager=%d\n", 
+    	accept_skip_pager, 
+	nws->skip_taskbar,
+	nws->skip_pager);
+
     RET(!(nws->skip_taskbar || (accept_skip_pager && nws->skip_pager)));
 }
 
@@ -132,28 +155,13 @@ static int
 accept_net_wm_window_type(net_wm_window_type *nwwt)
 {
     ENTER;
+    DBG("desktop=%d dock=%d splash=%d\n", nwwt->desktop, nwwt->dock, nwwt->splash);
+
     RET(!(nwwt->desktop || nwwt->dock || nwwt->splash));
 }
 
 
 
-
-static void
-get_wmclass(task *tk)
-{
-    ENTER;
-    if (tk->ch.res_name)
-        XFree(tk->ch.res_name);
-    if (tk->ch.res_class)
-        XFree(tk->ch.res_class);
-    if (!XGetClassHint (gdk_display, tk->win, &tk->ch)) 
-        tk->ch.res_class = tk->ch.res_name = NULL;
-    RET();
-}
-
-
-
-
 inline static void
 tk_free_names(task *tk)
 {    
@@ -162,26 +170,38 @@ tk_free_names(task *tk)
     DBG("tk->iname %s\n", tk->iname);
     g_free(tk->name);
     g_free(tk->iname);
-
     tk->name = tk->iname = NULL;
     RET();
 }
 
+static void
+tk_get_names(task *tk)
+{
+    char *name;
+    
+    ENTER;
+    tk_free_names(tk);
+    name = get_utf8_property(tk->win,  a_NET_WM_NAME);
+    DBG("a_NET_WM_NAME:%s\n", name);
+    if (!name) {
+        name = get_textproperty(tk->win,  XA_WM_NAME);
+        DBG("XA_WM_NAME:%s\n", name);
+    }
+    if (name) {		
+	tk->name = g_strdup_printf(" %s ", name);
+	tk->iname = g_strdup_printf("[%s]", name);
+	g_free(name);
+    }
+    RET();
+}
+
 static void
 tk_set_names(task *tk)
 {
     char *name;
     
     ENTER;
-    tk_free_names(tk);
-    name = get_textproperty(tk->win,  XA_WM_NAME);
-    DBG("utf name: %s\n", name);
-    if (name) {		
-	tk->name = g_strdup_printf(" %s ", name);
-	tk->iname = g_strdup_printf("[%s]", name);
-	g_free(name);
-        name = tk->iconified ? tk->iname : tk->name;
-    } 
+    name = tk->iconified ? tk->iname : tk->name;
     gtk_label_set_text(GTK_LABEL(tk->label), name);
     if (tk->tb->tooltips)
         gtk_tooltips_set_tip(tk->tb->tips, tk->button, tk->name, NULL);
@@ -203,9 +223,12 @@ del_task (taskbar * tb, task *tk, int hdel)
 {
     ENTER;
     DBG("deleting(%d)  %08x %s\n", hdel, tk->win, tk->name);
-    gtk_widget_destroy(tk->eb);
-    tb->num_tasks--; 
-    tk_free_names(tk);	    
+    if( tk->flash_timeout )
+        g_source_remove( tk->flash_timeout );
+    gtk_widget_destroy(tk->button);
+    tb->num_tasks--;
+    if (!tb->icons_only)
+        tk_free_names(tk);	    
     if (tb->focused == tk)
         tb->focused = NULL;
     if (hdel)
@@ -367,11 +390,13 @@ argbdata_to_pixdata (gulong *argb_data, int len)
 
     ENTER;
     ret = p = g_new (guchar, len * 4);
+    if (!ret)
+        RET(NULL);
     /* One could speed this up a lot. */
     i = 0;
     while (i < len) {
-        guint argb;
-        guint rgba;
+        guint32 argb;
+        guint32 rgba;
       
         argb = argb_data[i];
         rgba = (argb << 8) | (argb >> 24);
@@ -400,9 +425,24 @@ get_netwm_icon(Window tkwin, int iw, int ih)
 
     ENTER;
     data = get_xaproperty(tkwin, a_NET_WM_ICON, XA_CARDINAL, &n);
-    DBG("icon size = %d data = %p\n", n, data);
+    DBG("icon size = %d data = %p w=%ld h=%ld\n", n, data, data[0], data[1]);
+
+    if (0) {
+        gulong *tmp;
+        int len;
+        
+        len = n/sizeof(gulong);
+        tmp = data;
+        while (len > 2) {
+            int size = tmp[0] * tmp[1];
+            DBG("sub-icon: %dx%d %d bytes\n", tmp[0], tmp[1], size * 4);
+            len -= size + 2;
+            tmp += size;
+        }
+    }
+    
     if (data) {
-        if (n > 2) {
+        if (n > 2*sizeof(guint32)) {
             guchar *p;
             GdkPixbuf *src;
             int w, h;
@@ -410,7 +450,8 @@ get_netwm_icon(Window tkwin, int iw, int ih)
             w = data[0];
             h = data[1];
             p = argbdata_to_pixdata(data + 2, w * h);
-
+            if (!p)
+                RET(NULL);
             src = gdk_pixbuf_new_from_data (p, GDK_COLORSPACE_RGB, TRUE,
                   8, w, h, w * 4, free_pixels, NULL);
             if (src == NULL)
@@ -429,11 +470,13 @@ get_wm_icon(Window tkwin, int iw, int ih)
     XWMHints *hints;
     Pixmap xpixmap = None, xmask = None;
     Window win;
-    unsigned int w, h, d;
+    unsigned int w, h;
+    int sd;
     GdkPixbuf *ret, *masked, *pixmap, *mask = NULL;
  
     ENTER;
-    hints = (XWMHints *) get_xaproperty (tkwin, XA_WM_HINTS, XA_WM_HINTS, 0);
+    hints = XGetWMHints(GDK_DISPLAY(), tkwin);
+    DBG("\nwm_hints %s\n", hints ? "ok" : "failed");
     if (!hints)
         RET(NULL);
     
@@ -441,13 +484,15 @@ get_wm_icon(Window tkwin, int iw, int ih)
         xpixmap = hints->icon_pixmap;
     if ((hints->flags & IconMaskHint))
         xmask = hints->icon_mask;
-
-    XFree (hints);
+    DBG("flag=%ld xpixmap=%lx flag=%ld xmask=%lx\n", (hints->flags & IconPixmapHint), xpixmap,
+         (hints->flags & IconMaskHint),  xmask);
+    XFree(hints);
     if (xpixmap == None)
         RET(NULL);
     
-    if (!XGetGeometry (GDK_DISPLAY(), xpixmap, &win, &d, &d, &w, &h, &d, &d)) {
-        ERR("XGetGeometry failed\n");
+    if (!XGetGeometry (GDK_DISPLAY(), xpixmap, &win, &sd, &sd, &w, &h,
+              (guint *)&sd, (guint *)&sd)) {
+        DBG("XGetGeometry failed for %x pixmap\n", (unsigned int)xpixmap);
         RET(NULL);
     }
     DBG("tkwin=%x icon pixmap w=%d h=%d\n", tkwin, w, h);
@@ -455,7 +500,7 @@ get_wm_icon(Window tkwin, int iw, int ih)
     if (!pixmap)
         RET(NULL);
     if (xmask != None && XGetGeometry (GDK_DISPLAY(), xmask,
-              &win, &d, &d, &w, &h, &d, &d)) {
+              &win, &sd, &sd, &w, &h, (guint *)&sd, (guint *)&sd)) {
         mask = _wnck_gdk_pixbuf_get_from_pixmap (NULL, xmask, 0, 0, 0, 0, w, h);
         
         if (mask) {
@@ -487,26 +532,78 @@ tk_update_icon (taskbar *tb, task *tk, Atom a)
     GdkPixbuf *pixbuf;
     
     ENTER;
-    g_assert ((tb != NULL) && (tk != NULL));
-    g_return_if_fail(tk != NULL);
-
+    DBG("%lx: ", tk->win);
     pixbuf = tk->pixbuf;
     if (a == a_NET_WM_ICON || a == None) {
         tk->pixbuf = get_netwm_icon(tk->win, tb->iconsize, tb->iconsize);
         tk->using_netwm_icon = (tk->pixbuf != NULL);
+        DBGE("netwm_icon=%d ", tk->using_netwm_icon);
     }
-    if (!tk->using_netwm_icon) 
+    if (!tk->using_netwm_icon) {
         tk->pixbuf = get_wm_icon(tk->win, tb->iconsize, tb->iconsize);
-    if (!tk->pixbuf)
+        DBGE("wm_icon=%d ", (tk->pixbuf != NULL));
+    }
+    if (!tk->pixbuf) {
         tk->pixbuf = get_generic_icon(tb); // always exists
+        DBGE("generic_icon=1");
+    }
     if (pixbuf != tk->pixbuf) {
         if (pixbuf)
             g_object_unref(pixbuf);
     }
+    DBGE("\n");
     RET();
 }
 
+static gboolean 
+on_flash_win( task *tk )
+{
+    tk->flash_state = !tk->flash_state;
+    gtk_widget_set_state(tk->button,
+          tk->flash_state ? GTK_STATE_SELECTED : tk->tb->normal_state);
+    gtk_widget_queue_draw(tk->button);
+    return TRUE;
+}
 
+static void
+tk_flash_window( task *tk )
+{
+    gint interval;
+    tk->flash = 1;
+    tk->flash_state = !tk->flash_state;
+    if (tk->flash_timeout)
+        return;
+    g_object_get( gtk_widget_get_settings(tk->button),
+          "gtk-cursor-blink-time", &interval, NULL );
+    tk->flash_timeout = g_timeout_add(interval, (GSourceFunc)on_flash_win, tk);
+}
+
+static void
+tk_unflash_window( task *tk )
+{
+    tk->flash = tk->flash_state = 0;
+    if (tk->flash_timeout) {
+        g_source_remove(tk->flash_timeout);
+        tk->flash_timeout = 0;
+    }
+}
+
+static void
+tk_raise_window( task *tk, guint32 time )
+{
+    if (tk->desktop != -1 && tk->desktop != tk->tb->cur_desk){
+        Xclimsg(GDK_ROOT_WINDOW(), a_NET_CURRENT_DESKTOP, tk->desktop, 0, 0, 0, 0);
+        XSync (gdk_display, False);
+    }
+    if(use_net_active) {
+        Xclimsg(tk->win, a_NET_ACTIVE_WINDOW, 2, time, 0, 0, 0);
+    }
+    else {
+        XRaiseWindow (GDK_DISPLAY(), tk->win);
+        XSetInputFocus (GDK_DISPLAY(), tk->win, RevertToNone, CurrentTime);
+    }
+    DBG("XRaiseWindow %x\n", tk->win);
+}
 
 static void
 tk_callback_leave( GtkWidget *widget, task *tk)
@@ -527,6 +624,42 @@ tk_callback_enter( GtkWidget *widget, task *tk )
     RET();
 }
 
+static gboolean 
+delay_active_win(task* tk)
+{
+    tk_raise_window(tk, CurrentTime);
+    tk->tb->dnd_activate = 0;
+    return FALSE;
+}
+
+static gboolean
+tk_callback_drag_motion( GtkWidget *widget, 
+      GdkDragContext *drag_context,
+      gint x, gint y,
+      guint time, task *tk)
+{
+    /* prevent excessive motion notification */
+    if (!tk->tb->dnd_activate) {
+        tk->tb->dnd_activate = g_timeout_add(DRAG_ACTIVE_DELAY, 
+              (GSourceFunc)delay_active_win, tk);
+    }
+    gdk_drag_status (drag_context,0,time);
+    return TRUE;
+}
+
+static void
+tk_callback_drag_leave (GtkWidget *widget, 
+      GdkDragContext *drag_context,
+      guint time, task *tk)
+{
+    if (tk->tb->dnd_activate) {
+        g_source_remove(tk->tb->dnd_activate);
+        tk->tb->dnd_activate = 0;
+    }
+    return;
+}
+
+#if 0
 static gboolean
 tk_callback_expose(GtkWidget *widget, GdkEventExpose *event, task *tk)
 {
@@ -537,13 +670,20 @@ tk_callback_expose(GtkWidget *widget, GdkEventExpose *event, task *tk)
         gtk_widget_set_state(widget, state);
         gtk_widget_queue_draw(widget);
     } else {
-
-        gtk_paint_box (widget->style, widget->window,
-              state, 
-              (tk->focused) ? GTK_SHADOW_IN : GTK_SHADOW_OUT,
-              &event->area, widget, "button",
-              widget->allocation.x, widget->allocation.y,
-              widget->allocation.width, widget->allocation.height);
+        if( ! tk->flash || 0 == tk->flash_state ) {
+            gtk_paint_box (widget->style, widget->window,
+                  state, 
+                  (tk->focused) ? GTK_SHADOW_IN : GTK_SHADOW_OUT,
+                  &event->area, widget, "button",
+                  widget->allocation.x, widget->allocation.y,
+                  widget->allocation.width, widget->allocation.height);
+        } else {
+            gdk_draw_rectangle( widget->window, 
+                                widget->style->bg_gc[GTK_STATE_SELECTED],
+                                TRUE, 0, 0,
+                                widget->allocation.width,
+                                widget->allocation.height );
+        }
         /*
         _gtk_button_paint(GTK_BUTTON(widget), &event->area, state,
               (tk->focused) ? GTK_SHADOW_IN : GTK_SHADOW_OUT,
@@ -553,19 +693,32 @@ tk_callback_expose(GtkWidget *widget, GdkEventExpose *event, task *tk)
     }
     RET(FALSE);
 }
+#endif
 
-#if 0
-static void
-tk_callback_state_changed (GtkWidget *widget, GtkStateType state, task *tk)
+static gint
+tk_callback_scroll_event (GtkWidget *widget, GdkEventScroll *event, task *tk)
 {
-    GtkStateType reqstate;
     ENTER;
-    reqstate = (tk->focused) ? tk->tb->focused_state : tk->tb->normal_state;
-    if (state != reqstate)
-        gtk_widget_set_state(widget, reqstate);
-    RET();
+    if (event->direction == GDK_SCROLL_UP) {
+        GdkWindow *gdkwindow;
+        
+        gdkwindow = gdk_xid_table_lookup (tk->win);
+        if (gdkwindow)
+            gdk_window_show (gdkwindow);
+        else
+            XMapRaised (GDK_DISPLAY(), tk->win);                           
+        XSetInputFocus (GDK_DISPLAY(), tk->win, RevertToNone, CurrentTime);
+        DBG("XMapRaised  %x\n", tk->win);
+    } else if (event->direction == GDK_SCROLL_DOWN) {
+        DBG("tb->ptk = %x\n", (tk->tb->ptk) ? tk->tb->ptk->win : 0);
+        XIconifyWindow (GDK_DISPLAY(), tk->win, DefaultScreen(GDK_DISPLAY()));
+        DBG("XIconifyWindow %x\n", tk->win);
+    }
+    
+    XSync (gdk_display, False);
+    RET(TRUE);
 }
-#endif 
+
 
 static gboolean
 tk_callback_button_release_event(GtkWidget *widget, GdkEventButton *event, task *tk)
@@ -576,15 +729,19 @@ tk_callback_button_release_event(GtkWidget *widget, GdkEventButton *event, task
     DBG("win=%x\n", tk->win);
     if (event->button == 1) {
         if (tk->iconified)    {
-            GdkWindow *gdkwindow;
-
-            gdkwindow = gdk_xid_table_lookup (tk->win);
-            if (gdkwindow)
-                gdk_window_show (gdkwindow);
-            else 
-                XMapRaised (GDK_DISPLAY(), tk->win);                           
-            XSync (GDK_DISPLAY(), False);
-            DBG("XMapRaised  %x\n", tk->win);
+	    if(use_net_active) {
+		Xclimsg(tk->win, a_NET_ACTIVE_WINDOW, 2, event->time, 0, 0, 0);
+	    } else {
+		GdkWindow *gdkwindow;
+		
+		gdkwindow = gdk_xid_table_lookup (tk->win);
+		if (gdkwindow)
+		    gdk_window_show (gdkwindow);
+		else 
+		    XMapRaised (GDK_DISPLAY(), tk->win);                           
+		XSync (GDK_DISPLAY(), False);
+		DBG("XMapRaised  %x\n", tk->win);
+	    }
         } else {
             DBG("tb->ptk = %x\n", (tk->tb->ptk) ? tk->tb->ptk->win : 0);
             if (tk->focused || tk == tk->tb->ptk) {
@@ -592,15 +749,7 @@ tk_callback_button_release_event(GtkWidget *widget, GdkEventButton *event, task
                 XIconifyWindow (GDK_DISPLAY(), tk->win, DefaultScreen(GDK_DISPLAY()));
                 DBG("XIconifyWindow %x\n", tk->win);
             } else {
-                if (tk->desktop != -1 && tk->desktop != tk->tb->cur_desk) {
-                    //goto tk->desktop
-                    Xclimsg(GDK_ROOT_WINDOW(), a_NET_CURRENT_DESKTOP, tk->desktop, 0, 0, 0, 0);
-                    XSync (gdk_display, False);
-                }
-                XRaiseWindow (GDK_DISPLAY(), tk->win);
-                XSetInputFocus (GDK_DISPLAY(), tk->win, RevertToNone, CurrentTime);
-                DBG("XRaiseWindow %x\n", tk->win);
-
+                tk_raise_window( tk, event->time );
             }
         }
     } else if (event->button == 2) {
@@ -614,7 +763,9 @@ tk_callback_button_release_event(GtkWidget *widget, GdkEventButton *event, task
         DBG("XLowerWindow %x\n", tk->win);
         */
         tk->tb->menutask = tk;
-        gtk_menu_popup (GTK_MENU (tk->tb->menu), NULL, NULL, NULL, NULL,
+        gtk_menu_popup (GTK_MENU (tk->tb->menu), NULL, NULL,
+              (GtkMenuPositionFunc)menu_pos,
+              the_panel->orientation == ORIENT_HORIZ ? NULL : widget, 
               event->button, event->time);
         
     }
@@ -634,15 +785,15 @@ tk_update(gpointer key, task *tk, taskbar *tb)
               (tk->focused) ? tb->focused_state : tb->normal_state);
         gtk_widget_queue_draw(tk->button);
         //_gtk_button_set_depressed(GTK_BUTTON(tk->button), tk->focused);
-	gtk_widget_show(tk->eb);
+	gtk_widget_show(tk->button);
 
         if (tb->tooltips) {
-            //DBG2("tip %x %s\n", tk->win, tk->name);
+            //DBG("tip %x %s\n", tk->win, tk->name);
             gtk_tooltips_set_tip(tb->tips, tk->button, tk->name, NULL);
         }
 	RET();
     }    
-    gtk_widget_hide(tk->eb);
+    gtk_widget_hide(tk->button);
     RET();
 }
 
@@ -668,7 +819,7 @@ static void
 tk_build_gui(taskbar *tb, task *tk)
 {
     GtkWidget *w1;
-    
+
     ENTER;
     g_assert ((tb != NULL) && (tk != NULL));
 
@@ -678,55 +829,66 @@ tk_build_gui(taskbar *tb, task *tk)
      *
      * Do not change event mask to gtk windows spwaned by this gtk client
      * this breaks gtk internals */
-    if (!gdk_window_lookup(tk->win))
+    if (!FBPANEL_WIN(tk->win))
         XSelectInput (GDK_DISPLAY(), tk->win, PropertyChangeMask | StructureNotifyMask);
 
     /* button */
-    tk->eb = gtk_event_box_new();
     tk->button = gtk_button_new();
     gtk_widget_show(tk->button);
     gtk_container_set_border_width(GTK_CONTAINER(tk->button), 0);
     gtk_widget_add_events (tk->button, GDK_BUTTON_RELEASE_MASK );
     g_signal_connect(G_OBJECT(tk->button), "button_release_event",
-         G_CALLBACK(tk_callback_button_release_event), (gpointer)tk);  
+          G_CALLBACK(tk_callback_button_release_event), (gpointer)tk);  
     g_signal_connect_after (G_OBJECT (tk->button), "leave",
-         G_CALLBACK (tk_callback_leave), (gpointer) tk);    
+          G_CALLBACK (tk_callback_leave), (gpointer) tk);    
     g_signal_connect_after (G_OBJECT (tk->button), "enter",
-         G_CALLBACK (tk_callback_enter), (gpointer) tk);
-    g_signal_connect_after (G_OBJECT (tk->button), "expose-event",
-          G_CALLBACK (tk_callback_expose), (gpointer) tk);
-    //g_signal_connect_after (G_OBJECT (tk->button), "state-changed",
-    //     G_CALLBACK (tk_callback_state_changed), (gpointer) tk);
+          G_CALLBACK (tk_callback_enter), (gpointer) tk);
+    gtk_drag_dest_set( tk->button, 0, NULL, 0, 0);
+    g_signal_connect (G_OBJECT (tk->button), "drag-motion",
+          G_CALLBACK (tk_callback_drag_motion), (gpointer) tk);
+    g_signal_connect (G_OBJECT (tk->button), "drag-leave",
+          G_CALLBACK (tk_callback_drag_leave), (gpointer) tk);
+    if (tb->use_mouse_wheel)
+    	g_signal_connect_after(G_OBJECT(tk->button), "scroll-event",
+              G_CALLBACK(tk_callback_scroll_event), (gpointer)tk);	  
+
+ 
     /* pix and name */
-    w1 = tb->plug->panel->my_box_new(FALSE, 1);
+    w1 = gtk_hbox_new(FALSE, 1);
     gtk_container_set_border_width(GTK_CONTAINER(w1), 0);
 
     /* pix */
-    //get_wmclass(tk);
     tk_update_icon(tb, tk, None);
-    tk->image = gtk_image_new_from_pixbuf(tk->pixbuf );
+    tk->image = gtk_image_new_from_pixbuf(tk->pixbuf);
     gtk_widget_show(tk->image);
-    gtk_box_pack_start(GTK_BOX(w1), tk->image, FALSE, FALSE, 1);
+    gtk_box_pack_start(GTK_BOX(w1), tk->image, FALSE, FALSE, 0);
 
     /* name */
-    tk->label = gtk_label_new(tk->iconified ? tk->iname : tk->name);
-    gtk_label_set_justify(GTK_LABEL(tk->label), GTK_JUSTIFY_LEFT);
-    if (!tb->icons_only)
+    if (!tb->icons_only) {
+        tk->label = gtk_label_new(tk->iconified ? tk->iname : tk->name);
+        //gtk_label_set_justify(GTK_LABEL(tk->label), GTK_JUSTIFY_LEFT);
+        gtk_label_set_ellipsize(GTK_LABEL(tk->label), PANGO_ELLIPSIZE_END);
+        gtk_misc_set_alignment(GTK_MISC(tk->label), 0.0, 0.5); 
         gtk_widget_show(tk->label);
-    gtk_box_pack_start(GTK_BOX(w1), tk->label, FALSE, TRUE, 0);    
+        gtk_box_pack_start(GTK_BOX(w1), tk->label, TRUE, TRUE, 0);
+    }
     gtk_widget_show(w1);
     gtk_container_add (GTK_CONTAINER (tk->button), w1);
 
-    gtk_container_add (GTK_CONTAINER (tk->eb), tk->button);
-    gtk_box_pack_start(GTK_BOX(tb->bar), tk->eb, FALSE, TRUE, 0);
+    //gtk_container_add (GTK_CONTAINER (tk->eb), tk->button);
+    gtk_box_pack_start(GTK_BOX(tb->bar), tk->button, FALSE, TRUE, 0);
     GTK_WIDGET_UNSET_FLAGS (tk->button, GTK_CAN_FOCUS);    
     GTK_WIDGET_UNSET_FLAGS (tk->button, GTK_CAN_DEFAULT);
     
-    gtk_widget_show(tk->eb);
+    gtk_widget_show(tk->button);
     if (!task_visible(tb, tk)) {
-        gtk_widget_hide(tk->eb);
+        gtk_widget_hide(tk->button);
     }
 
+    if (tk->urgency) {
+        /* Flash button for window with urgency hint */
+        tk_flash_window(tk);
+    }
     RET();
 }
 
@@ -760,11 +922,9 @@ tb_net_client_list(GtkWidget *widget, taskbar *tb)
     tb->wins = get_xaproperty (GDK_ROOT_WINDOW(), a_NET_CLIENT_LIST, XA_WINDOW, &tb->win_num);
     if (!tb->wins) 
 	RET();
-    //DBG("tb_net_list -----enter------\n");
     for (i = 0; i < tb->win_num; i++) {
         if ((tk = g_hash_table_lookup(tb->task_list, &tb->wins[i]))) {
             tk->refcount++;
-            //DBG("tb_net_list <...>: 0x%x %s\n", tk->win, tk->name);
         } else {
             net_wm_window_type nwwt;
             net_wm_state nws;
@@ -777,26 +937,31 @@ tb_net_client_list(GtkWidget *widget, taskbar *tb)
                 continue;
             
             tk = g_new0(task, 1);
-            tk->refcount++;
+            tk->refcount = 1;
             tb->num_tasks++;
             tk->win = tb->wins[i];
             tk->tb = tb;
-            tk->iconified = (get_wm_state (tk->win) == IconicState);
+            tk->iconified = (get_wm_state(tk->win) == IconicState);
             tk->desktop = get_net_wm_desktop(tk->win);
             tk->nws = nws;
             tk->nwwt = nwwt;
+            if( tb->use_urgency_hint && tk_has_urgency(tk)) {
+                tk->urgency = 1;
+            }
          
             tk_build_gui(tb, tk);
-            tk_set_names(tk);
+            if (!tb->icons_only) {
+                tk_get_names(tk);
+                tk_set_names(tk);
+            }
             g_hash_table_insert(tb->task_list, &tk->win, tk);
-            DBG("adding %08x(%p) %s\n", tk->win, gdk_window_lookup(tk->win), tk->name);
+            DBG("adding %08x(%p) %s\n", tk->win, FBPANEL_WIN(tk->win), tk->name);
         }
     }
     
     /* remove windows that arn't in the NET_CLIENT_LIST anymore */
     g_hash_table_foreach_remove(tb->task_list, (GHRFunc) tb_remove_stale_tasks, NULL);
     tb_display(tb);
-    //DBG("tb_net_list -----exit------\n");
     RET();
 }
 
@@ -872,12 +1037,31 @@ tb_net_active_window(GtkWidget *widget, taskbar *tb)
     RET();
 }
 
+/* For older Xlib headers */
+#ifndef XUrgencyHint
+#define XUrgencyHint (1 << 8)
+#endif
+
+static gboolean
+tk_has_urgency( task* tk )
+{
+    XWMHints* hints;
+    
+    tk->urgency = 0;
+    hints = XGetWMHints(GDK_DISPLAY(), tk->win);
+    if (hints) {
+        if (hints->flags & XUrgencyHint) /* Got urgency hint */
+            tk->urgency = 1;
+        XFree( hints );
+    }
+    return tk->urgency;
+}
+
 static void
 tb_propertynotify(taskbar *tb, XEvent *ev)
 {
     Atom at;
     Window win;
-
     
     ENTER;
     DBG("win=%x\n", ev->xproperty.window);
@@ -894,29 +1078,34 @@ tb_propertynotify(taskbar *tb, XEvent *ev)
 	    tb_display(tb);	
 	}  else if (at == XA_WM_NAME) {
             DBG("WM_NAME\n");
-	    tk_set_names(tk);
-	    //tk_display(tb, tk);
+            if (!tb->icons_only) {
+                tk_get_names(tk);
+                tk_set_names(tk);
+            }
 	}  else if (at == XA_WM_CLASS) {
             DBG("WM_CLASS\n");
-
-            //get_wmclass(tk);
 	} else if (at == a_WM_STATE)    {
             DBG("WM_STATE\n");
 	    /* iconified state changed? */
-	    tk->iconified = (get_wm_state (tk->win) == IconicState);
-            tk_set_names(tk);
-	    //tk_display(tb, tk);
+            if (!tb->icons_only) {
+                tk->iconified = (get_wm_state (tk->win) == IconicState);
+                tk_set_names(tk);
+            }
 	} else if (at == XA_WM_HINTS)	{
 	    /* some windows set their WM_HINTS icon after mapping */
 	    DBG("XA_WM_HINTS\n");
-            //get_wmclass(tk);
-            DBG("#0 %d\n", GDK_IS_PIXBUF (tk->pixbuf));
 	    tk_update_icon (tb, tk, XA_WM_HINTS);
-            DBG("#1 %d\n", GDK_IS_PIXBUF (tk->pixbuf));
 	    gtk_image_set_from_pixbuf (GTK_IMAGE(tk->image), tk->pixbuf);
-            DBG("#3 %d\n", GDK_IS_PIXBUF (tk->pixbuf));
-      	    //tk_display(tb, tk);
-	} else if (at == a_NET_WM_STATE) {
+            if (tb->use_urgency_hint) {
+                if (tk_has_urgency(tk)) {
+                    //tk->urgency = 1;
+                    tk_flash_window(tk);
+                } else {
+                    //tk->urgency = 0;
+                    tk_unflash_window(tk);
+                }
+            }
+        } else if (at == a_NET_WM_STATE) {
             net_wm_state nws;
 
 	    DBG("_NET_WM_STATE\n");
@@ -1001,22 +1190,29 @@ taskbar_make_menu(taskbar *tb)
     ENTER;
     menu = gtk_menu_new ();
 
-    mi = gtk_menu_item_new_with_label ("Raise");
+    mi = gtk_image_menu_item_new_with_label ("Raise");
+    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi),
+          gtk_image_new_from_stock(GTK_STOCK_GO_UP, GTK_ICON_SIZE_MENU));
     gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
     g_signal_connect(G_OBJECT(mi), "activate", (GCallback)menu_raise_window, tb);
     gtk_widget_show (mi);
 
-    mi = gtk_menu_item_new_with_label ("Iconify");
+    mi = gtk_image_menu_item_new_with_label ("Iconify");
+    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi),
+          gtk_image_new_from_stock(GTK_STOCK_UNDO, GTK_ICON_SIZE_MENU));
     gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
     g_signal_connect(G_OBJECT(mi), "activate", (GCallback)menu_iconify_window, tb);
     gtk_widget_show (mi);
 
+
+    mi = gtk_separator_menu_item_new();
+    gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
+    gtk_widget_show (mi);
+    
     /* we want this item to be farest from mouse pointer */
-    mi = gtk_menu_item_new_with_label ("Close Window");
-    if (tb->plug->panel->edge == EDGE_BOTTOM)
-        gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), mi);
-    else
-        gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
+    //mi = gtk_menu_item_new_with_label ("Close Window");
+    mi = gtk_image_menu_item_new_from_stock(GTK_STOCK_CLOSE, NULL);
+    gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
     g_signal_connect(G_OBJECT(mi), "activate", (GCallback)menu_close_window, tb);
     gtk_widget_show (mi);
 
@@ -1031,31 +1227,14 @@ taskbar_build_gui(plugin *p)
     GtkBarOrientation  bo;
     
     ENTER;
-    /*
-    tb->hbox = tb->plug->panel->my_box_new(FALSE, 0);
-    gtk_container_add (GTK_CONTAINER (p->pwid), tb->hbox);
-    gtk_widget_show(tb->hbox);
-    gtk_container_set_border_width(GTK_CONTAINER(tb->hbox), 0);
-
     bo = (tb->plug->panel->orientation == ORIENT_HORIZ) ? GTK_BAR_HORIZ : GTK_BAR_VERTICAL;
-    tb->bbox = gtk_bar_new(bo, 2);
-    gtk_bar_set_max_child_size(GTK_BAR(tb->bbox), tb->task_width_max);
-    gtk_widget_show(tb->bbox);
-    gtk_box_pack_start(GTK_BOX(tb->hbox), tb->bbox, TRUE, TRUE, 0);
-    */
-    bo = (tb->plug->panel->orientation == ORIENT_HORIZ) ? GTK_BAR_HORIZ : GTK_BAR_VERTICAL;
-    tb->bar = gtk_bar_new(bo, 0); 
-    if (tb->icons_only) {
-        gtk_bar_set_max_child_size(GTK_BAR(tb->bar),
-              GTK_WIDGET(p->panel->box)->allocation.height -2);        
-     } else
-        gtk_bar_set_max_child_size(GTK_BAR(tb->bar), tb->task_width_max);
+    tb->bar = gtk_bar_new(bo, tb->spacing);
+    gtk_bar_set_max_child_size(GTK_BAR(tb->bar), tb->task_width_max);
     gtk_container_add (GTK_CONTAINER (p->pwid), tb->bar);
     gtk_widget_show(tb->bar);
   
     tb->gen_pixbuf =  gdk_pixbuf_new_from_xpm_data((const char **)icon_xpm);
 
-    //XSelectInput (GDK_DISPLAY(), GDK_ROOT_WINDOW(), PropertyChangeMask);
     gdk_window_add_filter(NULL, (GdkFilterFunc)tb_event_filter, tb );
 
     g_signal_connect (G_OBJECT (fbev), "current_desktop",
@@ -1064,7 +1243,7 @@ taskbar_build_gui(plugin *p)
           G_CALLBACK (tb_net_active_window), (gpointer) tb);
     g_signal_connect (G_OBJECT (fbev), "number_of_desktops",
           G_CALLBACK (tb_net_number_of_desktops), (gpointer) tb);
-    g_signal_connect (G_OBJECT (fbev), "client_list_stacking",
+    g_signal_connect (G_OBJECT (fbev), "client_list",
           G_CALLBACK (tb_net_client_list), (gpointer) tb);
 
     tb->desk_num = get_net_number_of_desktops();
@@ -1079,6 +1258,24 @@ taskbar_build_gui(plugin *p)
     RET();
 }
 
+void net_active_detect()
+{
+    int nitens;
+    Atom *data;
+
+    data = get_xaproperty(GDK_ROOT_WINDOW(), a_NET_SUPPORTED, XA_ATOM, &nitens);
+    if (!data)
+	return;
+    
+    while (nitens > 0) 
+	if(data[--nitens]==a_NET_ACTIVE_WINDOW) {
+	    use_net_active = TRUE;
+            break;
+        }
+    
+    XFree(data);
+}
+
 static int
 taskbar_constructor(plugin *p)
 {
@@ -1091,16 +1288,13 @@ taskbar_constructor(plugin *p)
     gtk_widget_set_name(p->pwid, "taskbar");
     gtk_rc_parse_string(taskbar_rc);
     get_button_spacing(&req, GTK_CONTAINER(p->pwid), "");
+   
+    net_active_detect();
     
     tb = g_new0(taskbar, 1);
     tb->plug = p;
     p->priv = tb;
     
-    if (p->panel->orientation == ORIENT_HORIZ) {
-        tb->iconsize = GTK_WIDGET(p->panel->box)->allocation.height - req.height;
-        DBG("pwid height = %d\n", GTK_WIDGET(p->pwid)->allocation.height);
-    } else
-        tb->iconsize = 24;
     tb->topxwin           = p->panel->topxwin;
     tb->tooltips          = 1;
     tb->icons_only        = 0;
@@ -1112,6 +1306,9 @@ taskbar_constructor(plugin *p)
     tb->task_list         = g_hash_table_new(g_int_hash, g_int_equal);
     tb->focused_state     = GTK_STATE_ACTIVE;
     tb->normal_state      = GTK_STATE_NORMAL;
+    tb->spacing           = 1;
+    tb->use_mouse_wheel   = 1;
+    tb->use_urgency_hint  = 1;
     s.len = 256;
     while (get_line(p->fp, &s) != LINE_BLOCK_END) {
         if (s.type == LINE_NONE) {
@@ -1134,6 +1331,12 @@ taskbar_constructor(plugin *p)
             } else if (!g_ascii_strcasecmp(s.t[0], "MaxTaskWidth")) {
                 tb->task_width_max = atoi(s.t[1]);
                 DBG("task_width_max = %d\n", tb->task_width_max);
+            } else if (!g_ascii_strcasecmp(s.t[0], "spacing")) {
+                tb->spacing = atoi(s.t[1]);
+            } else if (!g_ascii_strcasecmp(s.t[0], "UseMouseWheel")) {
+                tb->use_mouse_wheel = str2num(bool_pair, s.t[1], 1);
+            } else if (!g_ascii_strcasecmp(s.t[0], "UseUrgencyHint")) {
+                tb->use_urgency_hint = str2num(bool_pair, s.t[1], 1);
             } else {
                 ERR( "taskbar: unknown var %s\n", s.t[0]);
                 goto error;
@@ -1142,7 +1345,16 @@ taskbar_constructor(plugin *p)
             ERR( "taskbar: illegal in this context %s\n", s.str);
             goto error;
         }
-    }  
+    }
+    if (p->panel->orientation == ORIENT_HORIZ) {
+        tb->iconsize = GTK_WIDGET(p->panel->box)->allocation.height - req.height;
+        DBG("pwid height = %d\n", GTK_WIDGET(p->pwid)->allocation.height);
+        if (tb->icons_only)
+            tb->task_width_max = tb->iconsize + req.width;
+    } else {
+        tb->iconsize = 24;
+        tb->task_width_max = tb->iconsize + req.height;
+    }
     taskbar_build_gui(p);
     tb_net_client_list(NULL, tb);
     tb_display(tb);
@@ -1166,6 +1378,7 @@ taskbar_destructor(plugin *p)
     g_signal_handlers_disconnect_by_func(G_OBJECT (fbev), tb_net_number_of_desktops, tb);
     g_signal_handlers_disconnect_by_func(G_OBJECT (fbev), tb_net_client_list, tb);   
     gdk_window_remove_filter(NULL, (GdkFilterFunc)tb_event_filter, tb );
+    g_hash_table_destroy(tb->task_list); 
     gtk_widget_destroy(tb->bar);
     gtk_widget_destroy(tb->menu);
   
@@ -1184,3 +1397,4 @@ plugin_class taskbar_plugin_class = {
     constructor : taskbar_constructor,
     destructor  : taskbar_destructor,
 };
+
